@@ -2,7 +2,7 @@
 #define _MKTEXLIB_H_
 
 #if defined(WIN32)
-#  ifdef _DLL
+#  ifdef MKTEX_DLL
 #    ifdef MAKE_MKTEX_DLL
 #      define MKTEXDLL __declspec( dllexport)
 #    else
