@@ -96,7 +96,6 @@
 #ifndef DEBUG     /* strange logic: makeindex uses DEBUG to force debugging */
 #define NO_DEBUG  /* kpathsea uses NO_DEBUG to suppress it.                 */
 #endif
-#include <kpathsea/c-auto.h>
 #include <kpathsea/config.h>
 #include <kpathsea/c-limits.h>
 #include <kpathsea/c-memstr.h>
@@ -105,6 +104,7 @@
 #include <kpathsea/progname.h>
 #include <kpathsea/tex-glyph.h>
 #include <kpathsea/expand.h>
+#include <c-auto.h>
 #endif
 
 #include    <stdio.h>
@@ -213,8 +213,16 @@
 #define RSQ ']'
 #define BSH '\\'
 
+#ifdef DOSISH
+#define ENV_SEPAR ';'
+#define DIR_DELIM '/'
+#define ALT_DIR_DELIM '\\'
+#define IS_DIR_DELIM(c) ((c) == DIR_DELIM || (c) == ALT_DIR_DELIM)
+#else  /* not DOSISH */
 #define ENV_SEPAR ':'
 #define DIR_DELIM '/'
+#define IS_DIR_DELIM(c) ((c) == DIR_DELIM)
+#endif /* not DOSISH */
 
 #ifndef SW_PREFIX			/* can override at compile time */
 #define SW_PREFIX  '-'
@@ -296,7 +304,7 @@ violations are possible, with core dumps, or worse, incorrect output,
 ensuing.
 ======================================================================*/
 
-#define ARABIC_MAX    5		/* maximum digits in an Arabic page */
+#define ARABIC_MAX    10	/* maximum digits in an Arabic page */
 				/* number field */
 
 #define ARGUMENT_MAX  1024	/* maximum length of sort or actual key */
@@ -376,7 +384,7 @@ typedef struct KFIELD
     char    *af[FIELD_MAX];		/* actual key */
     int     group;			/* key group */
     char    lpg[NUMBER_MAX];		/* literal page */
-    short   npg[PAGEFIELD_MAX];		/* page field array */
+    int     npg[PAGEFIELD_MAX];		/* page field array */
     short   count;			/* page field count */
     short   type;			/* page number type */
     char    *encap;			/* encapsulator */
