@@ -155,7 +155,8 @@ char *staticligkern[] = {
  */
 struct encoding *outencoding = 0 ;
 struct encoding *inencoding = 0 ;
-char *outenname, *inenname ;/* the file names for input and output encodings */
+char *outenname = NULL, 
+  *inenname = NULL;/* the file names for input and output encodings */
 int boundarychar = -1 ;     /* the boundary character */
 int ignoreligkern ;         /* do we look at ligkern info in the encoding? */
 /*
@@ -662,7 +663,7 @@ handlereencoding P1H(void) {
          }
       for (i=0; i<256; i++) {
          p = inencoding->vec[i] ;
-         if (p && *p && 0 != (ai = findadobe(p))) {
+         if (p && *p && strcmp(p, ".notdef") != 0 && 0 != (ai = findadobe(p))) {
             ai->adobenum = i ;
             adobeptrs[i] = ai ;
          }
@@ -709,7 +710,8 @@ assignchars P1H(void) {
  *   First, we assign all those that match perfectly.
  */
    for (i=0, p=outencoding->vec; i<256; i++, p++)
-      if ((ai=findadobe(*p)) && (ai->adobenum >= 0 || ai->pccs != NULL)) {
+      if (*p && strcmp(*p, ".notdef") != 0 &&
+          (ai=findadobe(*p)) && (ai->adobenum >= 0 || ai->pccs != NULL)) {
          if (ai->texnum >= 0)
             nexttex[i] = ai->texnum ; /* linked list */
          ai->texnum = i ;
@@ -1464,8 +1466,8 @@ writevpl P1H(void)
 #ifdef KPATHSEA
 void version P1C(FILE *, f)
 {
-  extern DllImport char *kpathsea_version_string;
-  fputs ("afm2tfm(k) (dvips(k) 5.82a) 8.1\n", f);
+  extern KPSEDLL char *kpathsea_version_string;
+  fputs ("afm2tfm(k) (dvips(k) 5.86a) 8.1\n", f);
   fprintf (f, "%s\n", kpathsea_version_string);
   fputs ("Copyright (C) 1998 Radical Eye Software.\n\
 There is NO warranty.  You may redistribute this software\n\
@@ -1495,7 +1497,7 @@ Primary author of afm2tfm: T. Rokicki; -k maintainer: K. Berry.\n", f);
 
 void usage P1C(FILE *, f)
 {
-   extern DllImport char *kpse_bug_address;
+   extern KPSEDLL char *kpse_bug_address;
    
    fputs ("Usage: afm2tfm FILE[.afm] [OPTION]... [FILE[.tfm]]\n", f);
    fputs (USAGE, f);
@@ -1516,6 +1518,8 @@ void usage P1C(FILE *, f)
 }
 #endif
 
+#define CHECKARG3 if (argc < 4) { usage(stderr); exit(1); }
+
 void
 openfiles P2C(int, argc, char **, argv)
 {
@@ -1563,6 +1567,7 @@ openfiles P2C(int, argc, char **, argv)
       switch (i) {
 case 'V': makevpl++ ;
 case 'v': makevpl++ ;
+         CHECKARG3
          (void)strcpy(outname, argv[3]) ;
 #ifdef KPATHSEA
 	 if (find_suffix(outname) == NULL)
@@ -1587,26 +1592,33 @@ case 'v': makevpl++ ;
 #endif
             error("! can't open vpl output file") ;
          break ;
-case 'e': if (sscanf(argv[3], "%f", &efactor)==0 || efactor<0.01)
+case 'e': CHECKARG3
+          if (sscanf(argv[3], "%f", &efactor)==0 || efactor<0.01)
             error("! Bad extension factor") ;
          efactorparam = argv[3] ;
          break ;
 case 'c':
+         CHECKARG3
          if (sscanf(argv[3], "%f", &capheight)==0 || capheight<0.01)
             error("! Bad small caps height") ;
          break ;
-case 's': if (sscanf(argv[3], "%f", &slant)==0)
+case 's':
+         CHECKARG3
+         if (sscanf(argv[3], "%f", &slant)==0)
             error("! Bad slant parameter") ;
          slantparam = argv[3] ;
          break ;
 case 'P':
 case 'p':
+         CHECKARG3
          inenname = argv[3] ;
          break ;
 case 'T':
+         CHECKARG3
          inenname = outenname = argv[3] ;
          break ;
 case 't':
+         CHECKARG3
          outenname = argv[3] ;
          break ;
 case 'O':
@@ -1656,7 +1668,7 @@ default: (void)fprintf(stderr, "Unknown option %s %s will be ignored.\n",
      p = find_suffix(outname);
    *(p-1) = 0;
 
-   p = (char *)basename(outname) ;
+   p = (char *)xbasename(outname) ;
    strcpy(tmpstr, p);        /* be careful, p and outname are overlapping */
    strcpy(outname, tmpstr);
 #else
@@ -1964,11 +1976,13 @@ struct encoding *readencoding P1C(char *, enc)
 /*
  *   This routine prints out the line that needs to be added to psfonts.map.
  */
+#ifndef VMCMS
 void conspsfonts() {
    (void)printf("%s %s", outname,
-#ifndef VMCMS
    fontname) ;
 #else /* VM/CMS: fontname is ascii, so we use ebfontname */
+void conspsfonts() {
+   (void)printf("%s %s", outname,
    ebfontname) ;
 #endif
    if (slantparam || efactorparam || inenname) {
