@@ -185,6 +185,7 @@ fontdesctype *fonthd[MAXFONTHD];/* list headers for included fonts of 1 name */
 int nextfonthd;                 /* next unused fonthd[] index */
 char xdig[256];                 /* table for reading hexadecimal digits */
 char banner[] = BANNER ;        /* our startup message */
+char banner2[] = BANNER2 ;      /* our second startup message */
 Boolean noenv = 0 ;             /* ignore PRINTER envir variable? */
 Boolean dopprescan = 0 ;        /* do we do a scan before the prescan? */
 extern int dontmakefont ;
@@ -279,14 +280,14 @@ static char *helparr[] = {
 "p # First page                     P s Load config.$s                  ",
 #endif
 "q*  Run quietly                                                        ",
-"r*  Reverse order of pages         R   Run securely                    ",
+"r*  Reverse order of pages         R*  Run securely                    ",
 "s*  Enclose output in save/restore S # Max section size in pages       ",
 "t s Paper format                   T c Specify desired page size       ",  
 "u s PS mapfile                     U*  Disable string param trick      ",
 "                                   V*  Send downloadable PS fonts as PK",
 "x # Override dvi magnification     X # Horizontal resolution           ",
 "y # Multiply by dvi magnification  Y # Vertical resolution             ",  
-"                                   Z*  Compress bitmap fonts           ",
+"z*  Hyper PS                       Z*  Compress bitmap fonts           ",
 /* "-   Interactive query of options", */
 "    # = number   f = file   s = string  * = suffix, `0' to turn off    ",
 "    c = comma-separated dimension pair (e.g., 3.2in,-32.1cm)           ", 0} ;
@@ -297,7 +298,7 @@ help P1C(int, status)
    char **p;
    FILE *f = status == 0 ? stdout : stderr;
 #ifdef KPATHSEA
-   extern DllImport char *kpse_bug_address;
+   extern KPSEDLL char *kpse_bug_address;
 #endif   
    for (p=helparr; *p; p++)
       fprintf (f, "%s\n", *p);
@@ -455,10 +456,7 @@ initialize P1H(void)
    disablecomments = 0 ;
    maxdrift = -1 ;
    vmaxdrift = -1 ;
-#ifdef KPATHSEA
    dontmakefont = !MAKE_TEX_PK_BY_DEFAULT;
-   kpse_set_program_enabled (kpse_pk_format, !dontmakefont, kpse_src_compile);
-#endif
 }
 /*
  *   This routine copies a string into the string `pool', safely.
@@ -483,7 +481,7 @@ newstring P1C(char *, s)
 void newoutname P1H(void) {
    static int seq = 0 ;
    static char *seqptr = 0 ;
-   char *p ;
+   char *p, *seqslash = 0 ;
 
    if (oname == 0 || *oname == 0)
       error("! need an output file name to specify separate files") ;
@@ -491,9 +489,15 @@ void newoutname P1H(void) {
       if (seqptr == 0) {
          oname = newstring(oname) ;
          seqptr = 0 ;
-         for (p = oname; *p; p++)
+         for (p = oname; *p; p++)    /* find last dot after last slash */
             if (*p == '.')
                seqptr = p + 1 ;
+            else if (*p == '/')
+               seqptr = 0 ;
+#ifdef DOSISH
+            else if (*p == '\\')
+               seqptr = 0 ;
+#endif
          if (seqptr == 0)
             seqptr = p ;
          nextstring += 5 ; /* make room for the number, up to five digits */
@@ -552,7 +556,8 @@ main P2C(int, argc, char **, argv)
    register sectiontype *sects ;
 
 #ifdef KPATHSEA
-   kpse_set_progname (argv[0]);
+   kpse_set_program_name (argv[0], "dvips");
+   kpse_set_program_enabled (kpse_pk_format, MAKE_TEX_PK_BY_DEFAULT, kpse_src_compile);
 #endif
    
 #ifdef __THINK__
@@ -602,18 +607,27 @@ main P2C(int, argc, char **, argv)
         help (0);
         exit (0);
       } else if (strcmp (argv[1], "--version") == 0) {
-        extern DllImport char *kpathsea_version_string;
-        puts ("dvips(k) 5.78");
+        extern KPSEDLL char *kpathsea_version_string;
+        puts ("dvips(k) 5.86e");
         puts (kpathsea_version_string);
-        puts ("Copyright (C) 1998 Radical Eye Software.\n\
+        puts ("Copyright (C) 2001 Radical Eye Software.\n\
 There is NO warranty.  You may redistribute this software\n\
 under the terms of the GNU General Public License\n\
 and the Dvips copyright.\n\
 For more information about these matters, see the files\n\
 named COPYING and dvips.h.\n\
-Primary author of Dvips: T. Rokicki; -k maintainer: K. Berry.");
+Primary author of Dvips: T. Rokicki; -k maintainer: T. Kacvinsky/ S. Rahtz.");
         exit (0);
       }
+      if (argc == 2 && strncmp(argv[1], "-?", 2) == 0) {
+         printf("%s %s\n", banner, banner2) ;
+         help(0);
+         exit(0);
+      }
+      if (argc == 2 && strncmp(argv[1], "-v", 2) == 0) {
+         printf("%s %s\n", banner, banner2) ;
+         exit(0);
+      }
    }
 #endif
 #endif
@@ -696,7 +710,6 @@ case 'f' :
                noenv = 1 ;
                sendcontrolD = 0 ;
                break ;
-               break ;
 case 'u' :
                {
                   extern char *psmapfile;
@@ -711,6 +724,7 @@ case 'u' :
                   else 
                      psmapfile = strdup(PSname) ; /* a cute small memory leak (just as in 'p' option handling in resident.c) */
                }
+               break ;
 case 'h' : case 'H' :
                if (*p == 0 && argv[i+1])
                   p = argv[++i] ;
@@ -732,7 +746,7 @@ case 'k':
                cropmarks = (*p != '0') ;
                break ;
 case 'R':
-               secure = 1 ;
+               secure = (*p != '0') ;
                break ;
 case 'S':
                if (*p == 0 && argv[i+1])
@@ -765,6 +779,10 @@ case 'o' :
                oname = p ;
                noenv = 1 ;
                sendcontrolD = 0 ;
+#if defined(MSDOS) || defined(OS2)
+                if (*oname && oname[strlen(oname)-1] == ':')
+                    oname[strlen(oname)-1] = 0;     /* strip ':' off */
+#endif
                break ;
 case 'O' :
                if (*p == 0 && argv[i+1])
@@ -872,7 +890,7 @@ case 't' :
                   paperfmt = p ;
                break ;
 case 'v':
-                printf ("%s", BANNER);
+                printf ("%s %s\n", banner, banner2);
                 exit (0);
                 break;
 case 'x' : case 'y' :
@@ -952,10 +970,12 @@ case 'P' :
                   if (*p == 0 && argv[i+1])
                      p = argv[++i] ;
                   printer = p ;
+                  xputenv("PRINTER", printer);
+                  xputenv("LPDEST", printer);
                   noenv = 1 ;
                   if (!getdefaults("")) {
                     /* If no config file, default the output name.  */
-                    oname = concat ("| lpr -P", printer);
+                    oname = "| lpr";
                   }
                   npapsiz = opapsiz ;
                   while (npapsiz && npapsiz->next)
@@ -1033,6 +1053,7 @@ default:
             strcpy(nextstring, p) ;
             strcat(nextstring, ".cfg") ;
 #else
+            xputenv("LPDEST", p);
             strcpy(nextstring, "config.") ;
             strcat(nextstring, p) ;
 #endif
@@ -1050,7 +1071,7 @@ default:
       }
       papsizes = (struct papsiz *)revlist((void *)papsizes) ;
       if (queryoptions != 0) {            /* get new options */
-         (void)fprintf(stderr, banner) ;
+         (void)fprintf(stderr, "%s %s\n", banner, banner2) ;
          help(1) ;
          queryargs();
          if (qargc == 1)
@@ -1088,7 +1109,7 @@ default:
    getpsinfo((char *)NULL) ;
    revpslists() ;
    if (!quiet)
-      (void)fprintf(stderr, banner) ;
+      (void)fprintf(stderr, "%s %s\n", banner, banner2) ;
    if (*iname) {
       dvifile = fopen(iname, READBIN) ;
 /*
@@ -1101,7 +1122,7 @@ default:
    }
    if (oname[0] == '-' && oname[1] == 0)
       oname[0] = 0 ;
-   if (*oname == 0 && ! filter) {
+   else if (*oname == 0 && ! filter) {
       oname = nextstring ;
 #ifndef VMCMS  /* get stuff before LAST "." */
       lastext = strlen(iname) - 1 ;
