@@ -13,8 +13,6 @@
    Makefile rule.  */
 #include "TEX-OR-MF-OR-MPd.h"
 
-#include <kpathsea/c-auto.h>
-#include <kpathsea/config.h>
 #include <kpathsea/c-ctype.h>
 #include <kpathsea/line.h>
 #include <kpathsea/readable.h>
@@ -22,8 +20,10 @@
 #include <kpathsea/absolute.h>
 
 #include <time.h> /* For `struct tm'.  */
-#ifndef WIN32
-extern struct tm *localtime ();
+#if defined (HAVE_SYS_TIME_H)
+#include <sys/time.h>
+#elif defined (HAVE_SYS_TIMEB_H)
+#include <sys/timeb.h>
 #endif
 
 #if defined(__STDC__)
@@ -32,14 +32,18 @@ extern struct tm *localtime ();
 
 #include <signal.h> /* Catch interrupts.  */
 
+#include <texmfmp-help.h>
+
 /* {tex,mf}d.h defines TeX, MF, INI, and other such symbols.
    Unfortunately there's no way to get the banner into this code, so
    just repeat the text.  */
 #ifdef TeX
 #if defined (eTeX)
 #include <etexdir/etexextra.h>
-#elif defined (PDFTeX)
+#elif defined (pdfTeX)
 #include <pdftexdir/pdftexextra.h>
+#elif defined (pdfeTeX)
+#include <pdfetexdir/pdfetexextra.h>
 #elif defined (Omega)
 #include <omegadir/omegaextra.h>
 #else
@@ -100,33 +104,44 @@ int argc;
 static string user_progname;
 
 /* The C version of what might wind up in DUMP_VAR.  */
-static string dump_name;
+static const_string dump_name;
+
+/* The filename for dynamic character translation, or NULL.  */
+string translate_filename;
+string default_translate_filename;
+
+/* Needed for --src-specials option. */
+static char *last_source_name;
+static int last_lineno;
+static boolean srcspecialsoption = false;
+static void parse_src_specials_option P1H(const_string);
 
 /* The main body of the WEB is transformed into this procedure.  */
-extern void mainbody P1H(void);
+extern TEXDLL void mainbody P1H(void);
 
-static void maybe_set_dump_default_from_input P1H(string *);
+static void maybe_parse_first_line P1H(void);
 static void parse_options P2H(int, string *);
 
-#if defined(WIN32)
-  /* if _DEBUG is not defined, these macros will result in nothing. */
-   SETUP_CRTDBG;
-   /* Set the debug-heap flag so that freed blocks are kept on the
-    linked list, to catch any inadvertent use of freed memory */
-   SET_CRT_DEBUG_FIELD( _CRTDBG_DELAY_FREE_MEM_DF );
+#ifdef MP
+/* name of TeX program to pass to makempx */
+static string mpost_tex_program = "";
+#endif
+
+#ifdef __STDC__
+#ifdef WIN32
+extern boolean bOem;
+string locale_name = ".ACP";
+#else
+string locale_name = "";
+#endif
 #endif
 
 /* The entry point: set up for reading the command line, which will
    happen in `topenin', then call the main body.  */
 
-int
-main P2C(int, ac,  string *, av)
+void TEXDLL
+maininit P2C(int, ac, string *, av)
 {
-#ifdef __EMX__
-  _wildcard (&ac, &av);
-  _response (&ac, &av);
-#endif
-
   /* Save to pass along to topenin.  */
   argc = ac;
   argv = av;
@@ -134,16 +149,17 @@ main P2C(int, ac,  string *, av)
   /* Must be initialized before options are parsed.  */
   interactionoption = 4;
 
-#if defined(__STDC__)
-  /* "" means: get value from env. var LC_ALL, LC_CTYPE, or LANG */
-  setlocale(LC_CTYPE, "");
-#endif
-
   /* If the user says --help or --version, we need to notice early.  And
      since we want the --ini option, have to do it before getting into
      the web (which would read the base file, etc.).  */
   parse_options (ac, av);
   
+#if defined(__STDC__)
+  /* Need to delay it because of win32 `-oem' option.  Default value ""
+     means: get value from env. var LC_ALL, LC_CTYPE, or LANG */
+  setlocale(LC_CTYPE, locale_name);
+#endif
+
   /* Do this early so we can inspect program_invocation_name and
      kpse_program_name below, and because we have to do this before
      any path searching.  */
@@ -152,10 +168,14 @@ main P2C(int, ac,  string *, av)
   /* If no dump default yet, and we're not doing anything special on
      this run, look at the first line of the main input file for a
      %&<dumpname> specifier.  */
-  if (!dump_name) {
-    maybe_set_dump_default_from_input (&dump_name);
+  if (!dump_name || !translate_filename) {
+    maybe_parse_first_line ();
+  }
+  /* Check whether there still is no translate_filename known.  If so,
+     use the default_translate_filename. */
+  if (!translate_filename) {
+    translate_filename = default_translate_filename;
   }
-  
   /* If we're preloaded, I guess everything is set up.  I don't really
      know any more, it's been so long since anyone preloaded.  */
   if (readyalready != 314159) {
@@ -229,13 +249,37 @@ main P2C(int, ac,  string *, av)
   if (!outputcomment) {
     outputcomment = kpse_var_value ("output_comment");
   }
+  if (!srcspecialsoption) {
+       /* Look in texmf.cnf for a value */
+       string src_opt = kpse_var_value("src_specials");
+       /* No value means no specials. */
+       if (src_opt != NULL) {
+         last_source_name = xstrdup("");
+         parse_src_specials_option(src_opt);
+       }
+  }
 #endif /* TeX */
+}
+
+#ifndef WIN32
+/* The entry point: set up for reading the command line, which will
+   happen in `topenin', then call the main body.  */
+
+int
+main P2C(int, ac,  string *, av)
+{
+#ifdef __EMX__
+  _wildcard (&ac, &av);
+  _response (&ac, &av);
+#endif
+
+  maininit(ac, av);
 
   /* Call the real main program.  */
   mainbody ();
   return EXIT_SUCCESS;
 } 
-
+#endif /* ! WIN32 */
 
 /* This is supposed to ``open the terminal for input'', but what we
    really do is copy command line arguments into TeX's or Metafont's
@@ -272,18 +316,16 @@ topenin P1H(void)
   /* Make `last' be one past the last non-blank character in `buffer'.  */
   /* ??? The test for '\r' should not be necessary.  */
   for (--last; last >= first
-       && ISBLANK (buffer[last]) && buffer[last] != '\r'; --last)
+       && ISBLANK (buffer[last]) && buffer[last] != '\r'; --last) 
     ;
   last++;
 
   /* One more time, this time converting to TeX's internal character
      representation.  */
 #ifndef Omega
-#ifdef NONASCII
   for (i = first; i < last; i++)
     buffer[i] = xord[buffer[i]];
 #endif
-#endif
 }
 
 /* IPC for TeX.  By Tom Rokicki for the NeXT; it makes TeX ship out the
@@ -331,7 +373,7 @@ ipc_make_name P1H(void)
   if (ipc_addr_len == 0) {
     string s = getenv ("HOME");
     if (s) {
-      ipc_addr = xmalloc (strlen (s) + 40);
+      ipc_addr = (struct sockaddr*)xmalloc (strlen (s) + 40);
       ipc_addr->sa_family = 0;
       ipc_name = ipc_addr->sa_data;
       strcpy (ipc_name, s);
@@ -438,6 +480,7 @@ ipcpage P1C(int, is_eof)
 {
   static boolean begun = false;
   unsigned len = 0;
+  unsigned i;
   string p = "";
 
   if (!begun) {
@@ -451,11 +494,12 @@ ipcpage P1C(int, is_eof)
     len = strstartar[outputfilename + 1 - 65535L] -
             strstartar[outputfilename - 65535L];
 #endif
-    name = xmalloc (len + 1);
+    name = (string)xmalloc (len + 1);
 #ifndef Omega
     strncpy (name, &strpool[strstart[outputfilename]], len);
 #else
-    strncpy (name, &strpool[strstartar[outputfilename - 65535L]], len);
+    for (i=0; i<len; i++)
+      name[i] =  strpool[i+strstartar[outputfilename - 65535L]];
 #endif
     name[len] = 0;
     
@@ -475,12 +519,8 @@ ipcpage P1C(int, is_eof)
 }
 #endif /* TeX && IPC */
 
-#if 0 /* TCX files are probably a bad idea, since they make TeX source
-         documents unportable.  Try the inputenc LaTeX package.  */
-#ifdef TeX
+#if defined (TeX) || defined (MF) || defined (MP)
 #ifndef Omega  /* TCX and Omega get along like sparks and gunpowder. */
-/* The filename for dynamic character translation, or NULL.  */
-static string translate_filename;
 
 /* Return the next number following START, setting POST to the following
    character, as in strtol.  Issue a warning and return -1 if no number
@@ -513,6 +553,7 @@ tcx_get_num P3C(unsigned, line_count,  string, start,  string *, post)
 /* Look for the character translation file FNAME along the same path as
    tex.pool.  If no suffix in FNAME, use .tcx (don't bother trying to
    support extension-less names for these files).  */
+/* Probably a new format ought to be introduced for these files. */
 
 static void
 read_char_translation_file P1H(void)
@@ -523,7 +564,7 @@ read_char_translation_file P1H(void)
   }
   orig_filename = translate_filename;
   translate_filename
-    = kpse_find_file (translate_filename, kpse_texpool_format, true);
+    = kpse_find_file (translate_filename, kpse_web2c_format, true);
   if (translate_filename) {
     string line;
     unsigned line_count = 0;
@@ -553,21 +594,12 @@ read_char_translation_file P1H(void)
           second = first; /* else make internal the same as external */
         }
 
-        /* Don't be confused if they happen to mention the same
-           character twice.  */
-        if (!isprintable[second]) {
-          /* If they mention a charcode, call it printable.  */
-          isprintable[second] = 1;
-
-          /* Before, the string pool contained either three or four
-             bytes as the representation for this character, e.g., `^^@'
-             or `^^ff'.  But by the time we're done it will contain just
-             one, the character itself.  */
-          charssavedbycharset += second < 128 ? 2 : 3;
-	}        
+        /* If they mention a charcode, call it printable.  */
+        isprintable[second] = 1;
       }
       free (line);
     }
+    xfclose(translate_file, translate_filename);
   } else {
     WARNING1 ("Could not open char translation file `%s'", orig_filename);
   }
@@ -591,30 +623,34 @@ setupcharset P1H(void)
      to boringly specify all the usual stuff.  It also means it can't
      override the usual stuff.  This is a good thing, because of the way
      we handle the string pool. */
+  for (c = 0; c <= 255; c++) {
+    xchr[c] = xord[c] = c;
+    isprintable[c] = (32 <= c && c <= 126);
+  }
+
+#if 0
+  /* Get value from cnf file/environment variable if the option gives
+     a special value.  */
+  if (translate_filename && STREQ (translate_filename, "-")) {
+    free (translate_filename);
+    translate_filename = kpse_var_value ("CHARTRANSLATE");
+  }
+#endif
+
+  /* The expansion is nonempty if the variable is set.  */
+  if (translate_filename) {
+    read_char_translation_file ();
+  } else {
+    /* Use the locale to adjust the isprintable array. */
     for (c = 0; c <= 255; c++) {
-      xchr[c] = xord[c] = c;
-      isprintable[c] = (32 <= c && c <= 126);
-    }
-  
-  /* We cannot allow the user to change the string table that is dumped
-     in a .fmt file, because our strategy for dynamic translation
-     assumes the dynamic changes can only decrease the number of
-     characters used.  See tex.ch.  */
-  if (!iniversion) {
-    /* Get value from cnf file/environment variable unless already
-       specified via an option.  */
-    if (!translate_filename) {
-      translate_filename = kpse_var_value ("TEXCHARTRANSLATE");
-    }
-    /* The expansion is nonempty if the variable is set.  */
-    if (translate_filename) {
-      read_char_translation_file ();
+      if (!isprintable[c] && isprint(c)) {
+        isprintable[c] = 1;
+      }
     }
   }
 }  
 #endif /* !Omega */
-#endif /* TeX [character translation] */
-#endif /* 0, no TCX files, please */
+#endif /* TeX || MF || MP [character translation] */
 
 /* Reading the options.  */
 
@@ -625,38 +661,45 @@ setupcharset P1H(void)
 
 /* SunOS cc can't initialize automatic structs, so make this static.  */
 static struct option long_options[]
-  = { { DUMP_OPTION,		1, 0, 0 },
-      { "help",                 0, 0, 0 },
-      { "ini",			0, (int *) &iniversion, 1 },
-      { "interaction",          1, 0, 0 },
-      { "kpathsea-debug",	1, 0, 0 },
-      { "progname",             1, 0, 0 },
-      { "version",              0, 0, 0 },
+  = { { DUMP_OPTION,              1, 0, 0 },
+      { "help",                   0, 0, 0 },
+      { "ini",                    0, &iniversion, 1 },
+      { "interaction",            1, 0, 0 },
+      { "kpathsea-debug",         1, 0, 0 },
+      { "progname",               1, 0, 0 },
+      { "version",                0, 0, 0 },
+      { "recorder",               0, &recorder_enabled, 1 },
 #ifdef TeX
 #ifdef IPC
-      { "ipc",			0, (int *) &ipcon, 1 },
-      { "ipc-start",		0, (int *) &ipcon, 2 },
+      { "ipc",                    0, &ipcon, 1 },
+      { "ipc-start",              0, &ipcon, 2 },
 #endif /* IPC */
 #ifndef Omega
-      { "mltex",		0, (int *) &mltexp, 1 },
+      { "mltex",                  0, &mltexp, 1 },
 #endif /* !Omega */
-      { "output-comment",	1, 0, 0 },
-      { "shell-escape",		0, (int *) &shellenabledp, 1 },
-#if 0 /* TCX files are probably a bad idea */
-#ifndef Omega
-#ifdef NONASCII
-      { "translate-file",	1, 0, 0 },
-#endif /* NONASCII */
-#endif /* !Omega */
-#endif /* 0 */
+      { "output-comment",         1, 0, 0 },
+      { "shell-escape",           0, &shellenabledp, 1 },
+      { "debug-format",           0, &debugformatfile, 1 },
+      { "src-specials",           2, 0, 0 },
 #endif /* TeX */
+#if defined (TeX) || defined (MF) || defined (MP)
+      { "file-line-error-style",  0, &filelineerrorstylep, 1 },
+#ifndef Omega
+      { "translate-file",         1, 0, 0 },
+      { "default-translate-file", 1, 0, 0 },
+#if defined(WIN32) && defined(OEM)
+      { "oem",                    0, 0, 0 },
+#endif
+#endif /* !Omega */
+#endif /* TeX || MF || MP */
 #if defined (TeX) || defined (MF)
-      { "mktex",                1, 0, 0 },
-      { "no-mktex",             1, 0, 0 },
+      { "mktex",                  1, 0, 0 },
+      { "no-mktex",               1, 0, 0 },
 #endif /* TeX or MF */
 #ifdef MP
-      { "T",			0, (int *) &troffmode, 1 },
-      { "troff",		0, (int *) &troffmode, 1 },
+      { "T",                      0, &troffmode, 1 },
+      { "troff",                  0, &troffmode, 1 },
+      { "tex",                    1, 0, 0 },
 #endif /* MP */
       { 0, 0, 0, 0 } };
 
@@ -668,13 +711,13 @@ parse_options P2C(int, argc,  string *, argv)
   int option_index;
 
   for (;;) {
-    g = getopt_long_only (argc, argv, "", long_options, &option_index);
+    g = getopt_long_only (argc, argv, "+", long_options, &option_index);
 
     if (g == -1) /* End of arguments, exit the loop.  */
       break;
 
     if (g == '?') { /* Unknown option.  */
-      usage (1, argv[0]);
+      usage (argv[0]);
     }
 
     assert (g == 0); /* We have no short option names.  */
@@ -686,7 +729,8 @@ parse_options P2C(int, argc,  string *, argv)
       user_progname = optarg;
 
     } else if (ARGUMENT_IS (DUMP_OPTION)) {
-      dump_name = user_progname = optarg;
+      dump_name = optarg;
+      if (!user_progname) user_progname = optarg;
       dumpoption = true;
 
 #ifdef TeX
@@ -697,16 +741,11 @@ parse_options P2C(int, argc,  string *, argv)
       } else {
         WARNING2 ("Comment truncated to 255 characters from %d. (%s)",
                   len, optarg);
-        outputcomment = xmalloc (256);
+        outputcomment = (string)xmalloc (256);
         strncpy (outputcomment, optarg, 255);
         outputcomment[255] = 0;
       }
 
-#if 0 /* TCX files are probably a bad idea.  */
-    } else if (ARGUMENT_IS ("translate-file")) {
-      translate_filename = optarg;
-#endif
-
 #ifdef IPC
     } else if (ARGUMENT_IS ("ipc-start")) {
       ipc_open_out ();
@@ -719,9 +758,33 @@ parse_options P2C(int, argc,  string *, argv)
             ipc_open_out ();
           }
         }
-     }
+      }
 #endif /* IPC */
+    } else if (ARGUMENT_IS ("src-specials")) {
+       last_source_name = xstrdup("");
+       /* Option `--src" without any value means `auto' mode. */
+       if (optarg == 0) {
+         insertsrcspecialeverypar = true;
+         insertsrcspecialauto = true;
+         srcspecialsoption = true;
+       } else {
+          parse_src_specials_option(optarg);
+       }
 #endif /* TeX */
+#if defined (TeX) || defined (MF) || defined (MP)
+#ifndef Omega
+    } else if (ARGUMENT_IS ("translate-file")) {
+      translate_filename = optarg;
+    } else if (ARGUMENT_IS ("default-translate-file")) {
+      default_translate_filename = optarg;
+#if defined(WIN32) && defined(OEM)
+    } else if (ARGUMENT_IS ("oem")) {
+      /* This should switch the locale to the current OEM code page */
+      locale_name = ".OCP";
+      bOem = true;
+#endif
+#endif /* !Omega */
+#endif /* TeX || MF || MP */
 
 #if defined (TeX) || defined (MF)
     } else if (ARGUMENT_IS ("mktex")) {
@@ -730,7 +793,10 @@ parse_options P2C(int, argc,  string *, argv)
     } else if (ARGUMENT_IS ("no-mktex")) {
       kpse_maketex_option (optarg, false);
 #endif /* TeX or MF */
-
+#if defined (MP)
+    } else if (ARGUMENT_IS ("tex")) {
+      mpost_tex_program = optarg;
+#endif /* MP */
     } else if (ARGUMENT_IS ("interaction")) {
         /* These numbers match @d's in *.ch */
       if (STREQ (optarg, "batchmode")) {
@@ -746,12 +812,7 @@ parse_options P2C(int, argc,  string *, argv)
       }
       
     } else if (ARGUMENT_IS ("help")) {
-      string help = PROGRAM_HELP;
-#if defined (TeX) && defined (IPC)
-      /* Don't say the options exist unless they really do.  */
-      help = concat (help, TEX_IPC_HELP);
-#endif
-      usage (0, help);
+       usagehelp (PROGRAM_HELP);
 
     } else if (ARGUMENT_IS ("version")) {
       printversionandexit (BANNER, COPYRIGHT_HOLDER, AUTHOR);
@@ -760,6 +821,53 @@ parse_options P2C(int, argc,  string *, argv)
   }
 }
 
+#if defined(TeX)
+void 
+parse_src_specials_option P1C(const_string, opt_list)
+{
+  char * toklist = xstrdup(opt_list);
+  char * tok;
+  insertsrcspecialauto = false;
+  tok = strtok (toklist, ", ");
+  while (tok) {
+    if (strcmp (tok, "everypar") == 0
+        || strcmp (tok, "par") == 0
+        || strcmp (tok, "auto") == 0) {
+      insertsrcspecialauto = true;
+      insertsrcspecialeverypar = true;
+    } else if (strcmp (tok, "everyparend") == 0
+               || strcmp (tok, "parend") == 0)
+      insertsrcspecialeveryparend = true;
+    else if (strcmp (tok, "everycr") == 0
+             || strcmp (tok, "cr") == 0)
+      insertsrcspecialeverycr = true;
+    else if (strcmp (tok, "everymath") == 0
+             || strcmp (tok, "math") == 0)
+      insertsrcspecialeverymath = true;
+    else if (strcmp (tok, "everyhbox") == 0
+             || strcmp (tok, "hbox") == 0)
+      insertsrcspecialeveryhbox = true;
+    else if (strcmp (tok, "everyvbox") == 0
+             || strcmp (tok, "vbox") == 0)
+      insertsrcspecialeveryvbox = true;
+    else if (strcmp (tok, "everydisplay") == 0
+             || strcmp (tok, "display") == 0)
+      insertsrcspecialeverydisplay = true;
+    else if (strcmp (tok, "none") == 0) {
+      /* This one allows to reset an option that could appear in texmf.cnf */
+      insertsrcspecialauto = insertsrcspecialeverypar = 
+        insertsrcspecialeveryparend = insertsrcspecialeverycr = 
+        insertsrcspecialeverymath =  insertsrcspecialeveryhbox =
+        insertsrcspecialeveryvbox = insertsrcspecialeverydisplay = false;
+    } else {
+      WARNING1 ("Ignoring unknown argument `%s' to --src-specials", tok);
+    }
+    tok = strtok(0, ", ");
+  }
+  free(toklist);
+  srcspecialsoption = true;
+}
+#endif
 
 /* If the first thing on the command line (we use the globals `argv' and
    `optind') is a normal filename (i.e., does not start with `&' or
@@ -769,11 +877,20 @@ parse_options P2C(int, argc,  string *, argv)
    format are used.  */
 
 static void
-maybe_set_dump_default_from_input P1C(string *, dump_var)
+maybe_parse_first_line P1H(void)
 {
   char first_char = optind < argc ? argv[optind][0] : 0;
-  
-  if (first_char != '&' && first_char != '\\' && first_char) {
+  string parse_first_line = kpse_var_value ("parse_first_line");
+
+  /* Default to parsing the first line. */
+  if (!parse_first_line) parse_first_line = "t";
+
+  if ((*parse_first_line == 't'
+       || *parse_first_line == 'y'
+       || *parse_first_line == '1')
+      && first_char
+      && first_char != '&'
+      && first_char != '\\') {
     /* If the file can't be found, don't look too hard now.  We'll
        detect that it's missing in the normal course of things and give
        the error then.  */
@@ -782,66 +899,113 @@ maybe_set_dump_default_from_input P1C(string *, dump_var)
     if (f) {
       string first_line = read_line (f);
       xfclose (f, in_name);
-      
-      /* %&ini is special.  */
+
+      /* We deal with the general format "%&fmt --translate-file=tcx" */
+      /* The idea of using this format came from Wlodzimierz Bzyl
+         <matwb@monika.univ.gda.pl> */
       if (first_line && first_line[0] == '%' && first_line[1] == '&') {
-        if (strncmp (first_line + 1, "ini", 3) == 0) {
-          iniversion = true;
-        } else {
-          string f_name = concat (first_line+2, DUMP_EXT);
-          string d_name = kpse_find_file (f_name, DUMP_FORMAT, false);
-          if (d_name && kpse_readable_file (d_name)) {
-            dump_name = xstrdup (first_line + 2);
-            kpse_reset_program_name (dump_name);
-            /* Tell TeX/MF/MP we have a %&name line... */
-            dumpline = true;
+        /* Parse the first line into at most three space-separated parts. */
+        char *s;
+        char *part[4];
+        int npart;
+        char **parse;
+
+        for (s = first_line+2; ISBLANK(*s); ++s)
+          ;
+        npart = 0;
+        while (*s && npart != 3) {
+          part[npart++] = s;
+          while (*s && *s != ' ') s++;
+          while (*s == ' ') *s++ = '\0';
+        }
+        part[npart] = NULL;
+        parse = part;
+        /* Look at what we've got.  Very crude! */
+        if (*parse && **parse != '-') {
+          /* A format name */
+          if (dump_name) {
+              /* format already determined, do nothing. */
+          } else {
+            string f_name = concat (part[0], DUMP_EXT);
+            string d_name = kpse_find_file (f_name, DUMP_FORMAT, false);
+            if (d_name && kpse_readable_file (d_name)) {
+              dump_name = xstrdup (part[0]);
+              kpse_reset_program_name (dump_name);
+              /* Tell TeX/MF/MP we have a %&name line... */
+              dumpline = true;
+            }
+            free (f_name);
+          }
+          parse++;
+        }
+        /* The tcx stuff, if any.  Should we support the -translate-file
+           form as well as --translate-file?  */
+        if (*parse) {
+          if (translate_filename) {
+            /* TCX file already set, do nothing. */
+          } else if (STREQ (*parse, "--translate-file")) {
+            s = *(parse+1);
+          } else if (STREQ (*parse, "-translate-file")) {
+            s = *(parse+1);
+          } else if (STRNEQ (*parse, "--translate-file=", 17)) {
+            s = *parse+17;
+          } else if (STRNEQ (*parse, "-translate-file=", 16)) {
+            s = *parse+16;
+          }
+          /* Just set the name, no sanity checks here. */
+          if (s && *s) {
+            translate_filename = xstrdup(s);
           }
-          free (f_name);
         }
       }
-      
       if (first_line)
         free (first_line);
     }
   }
 }
 
-#if defined (TeX) || defined (MP)
-/* Return true if FNAME is acceptable as a name for \openout.  */
+/* Return true if FNAME is acceptable as a name for \openout, \openin, or
+   \input.  */
 
-boolean
-openoutnameok P1C(const_string, fname)
+static boolean
+opennameok P3C(const_string, fname, const_string, check_var,
+               const_string, default_choice)
 {
   /* We distinguish three cases:
-     'a' (any)        allows any file to be opened for writing.
+     'a' (any)        allows any file to be opened.
      'r' (restricted) means disallowing special file names.
      'p' (paranoid)   means being really paranoid: disallowing special file
                       names and restricting output files to be in or below
-                      the working directory or $TEXMFOUTPUT.
-     We default to "paranoid".
+                      the working directory or $TEXMFOUTPUT, while input files
+                      must be below the current directory, $TEXMFOUTPUT, or
+                      (implicitly) in the system areas.
+     We default to "paranoid".  The error messages from TeX will be somewhat
+     puzzling...
      This function contains several return statements...  */
 
-  const_string openout_any = kpse_var_value ("openout_any");
+  const_string open_choice = kpse_var_value (check_var);
 
-  if (openout_any && (*openout_any == 'a'
-                      || *openout_any == 'y'
-                      || *openout_any == '1'))
+  if (!open_choice) open_choice = default_choice;
+
+  if (*open_choice == 'a' || *open_choice == 'y' || *open_choice == '1')
     return true;
 
 #if defined (unix) && !defined (MSDOS)
   {
-    const_string base = basename (fname);
+    const_string base = xbasename (fname);
     /* Disallow .rhosts, .login, etc.  Allow .tex (for LaTeX).  */
-    if (*base == 0 || (*base == '.' && !STREQ (base, ".tex")))
+    if (base[0] == 0 ||
+        (base[0] == '.' && !IS_DIR_SEP(base[1]) && !STREQ (base, ".tex"))) {
+      fprintf(stderr, "%s: Not writing to %s (%s = %s).\n",
+              program_invocation_name, fname, check_var, open_choice);
       return false;
+    }
   }
 #else
   /* Other OSs don't have special names? */
 #endif
 
-  if (openout_any && (*openout_any == 'r'
-                      || *openout_any == 'n'
-                      || *openout_any == '0'))
+  if (*open_choice == 'r' || *open_choice == 'n' || *open_choice == '0')
     return true;
 
   /* Paranoia supplied by Charles Karney...  */
@@ -851,24 +1015,43 @@ openoutnameok P1C(const_string, fname)
        fname begins the TEXMFOUTPUT, and is followed by / */
     if (!texmfoutput || *texmfoutput == '\0'
         || fname != strstr (fname, texmfoutput)
-        || fname[strlen(texmfoutput)] != DIR_SEP)
+        || !IS_DIR_SEP(fname[strlen(texmfoutput)])) {
+      fprintf(stderr, "%s: Not writing to %s (%s = %s).\n",
+              program_invocation_name, fname, check_var, open_choice);
       return false;
+    }
   }
   /* For all pathnames, we disallow "../" at the beginning or "/../"
      anywhere.  */
-  if (fname[0] == '.' && fname[1] == '.' && fname[2] == DIR_SEP) {
+  if (fname[0] == '.' && fname[1] == '.' && IS_DIR_SEP(fname[2])) {
+    fprintf(stderr, "%s: Not writing to %s (%s = %s).\n",
+            program_invocation_name, fname, check_var, open_choice);
     return false;
   } else {
     const_string dotpair = strstr (fname, "..");
     /* If dotpair[2] == DIR_SEP, then dotpair[-1] is well-defined. */
-    if (dotpair && dotpair[2] == DIR_SEP && dotpair[-1] == DIR_SEP)
+    if (dotpair && IS_DIR_SEP(dotpair[2]) && IS_DIR_SEP(dotpair[-1])) {
+      fprintf(stderr, "%s: Not writing to %s (%s = %s).\n",
+              program_invocation_name, fname, check_var, open_choice);
       return false;
+    }
   }
 
   /* We passed all tests.  */
   return true;
 }
-#endif /* TeX or MP */
+
+boolean openinnameok P1C(const_string, fname)
+{
+    /* For input default to all. */
+    return opennameok (fname, "openin_any", "a");
+}
+
+boolean openoutnameok P1C(const_string, fname)
+{
+    /* For output, default to paranoid. */
+    return opennameok (fname, "openout_any", "p");
+}
 
 /* All our interrupt handler has to do is set TeX's or Metafont's global
    variable `interrupt'; then they will do everything needed.  */
@@ -949,6 +1132,27 @@ get_date_and_time P4C(integer *, minutes,  integer *, day,
   }
 }
 
+/*
+  Generating a better seed numbers
+  */
+integer
+getrandomseed()
+{
+#if defined (HAVE_GETTIMEOFDAY)
+  struct timeval tv;
+  gettimeofday(&tv, NULL);
+  return (tv.tv_usec + 1000000 * tv.tv_usec);
+#elif defined (HAVE_FTIME)
+  struct timeb tb;
+  ftime(&tb);
+  return (tb.millitm + 1000 * tb.time);
+#else
+  time_t clock = time ((time_t*)NULL);
+  struct tm *tmptr = localtime(&clock);
+  return (tmptr->tm_sec + 60*(tmptr->tm_min + 60*tmptr->tm_hour));
+#endif
+}
+
 /* Read a line of input as efficiently as possible while still looking
    like Pascal.  We set `last' to `first' and return `false' if we get
    to eof.  Otherwise, we return `true' and set last = first +
@@ -964,7 +1168,7 @@ input_line P1C(FILE *, f)
   while (last < bufsize && (i = getc (f)) != EOF && i != '\n' && i != '\r')
     buffer[last++] = i;
 
-  if (i == EOF && last == first)
+  if (i == EOF && errno != EINTR && last == first)
     return false;
 
   /* We didn't get the whole line because our buffer was too small.  */
@@ -981,7 +1185,8 @@ input_line P1C(FILE *, f)
 
   /* If next char is LF of a CRLF, read it.  */
   if (i == '\r') {
-    i = getc (f);
+    while ((i = getc (f)) == EOF && errno == EINTR)
+      ;
     if (i != '\n')
       ungetc (i, f);
   }
@@ -992,10 +1197,8 @@ input_line P1C(FILE *, f)
 
   /* Don't bother using xord if we don't need to.  */
 #ifndef Omega
-#ifdef NONASCII
   for (i = first; i <= last; i++)
      buffer[i] = xord[buffer[i]];
-#endif
 #endif
 
     return true;
@@ -1011,10 +1214,10 @@ static char *edit_value = EDITOR;
    actual filename starts; FNLENGTH is how long the filename is.  */
    
 void
-calledit (filename, fnstart, fnlength, linenumber)
-    ASCIIcode *filename;
-    poolpointer fnstart;
-    integer fnlength, linenumber;
+calledit P4C(packedASCIIcode *, filename,
+             poolpointer, fnstart,
+             integer, fnlength,
+             integer, linenumber)
 {
   char *temp, *command;
   char c;
@@ -1082,7 +1285,13 @@ calledit (filename, fnstart, fnlength, linenumber)
   *temp = 0;
 
   /* Execute the command.  */
+#ifdef WIN32
+  /* Win32 reimplementation of the system() command
+     provides opportunity to call it asynchronously */
+  if (win32_system(command, true) != 0 )
+#else
   if (system (command) != 0)
+#endif
     fprintf (stderr, "! Trouble executing `%s'.\n", command);
 
   /* Quit, since we found an error.  */
@@ -1225,7 +1434,7 @@ setupboundvariable P3C(integer *, var,  const_string, var_name,  integer, dflt)
     integer conf_val = atoi (expansion);
     /* It's ok if the cnf file specifies 0 for extra_mem_{top,bot}, etc.
        But negative numbers are always wrong.  */
-    if (conf_val < 0 || conf_val == 0 && dflt > 0) {
+    if (conf_val < 0 || (conf_val == 0 && dflt > 0)) {
       fprintf (stderr,
                "%s: Bad value (%ld) in texmf.cnf for %s, keeping %ld.\n",
                program_invocation_name,
@@ -1237,6 +1446,128 @@ setupboundvariable P3C(integer *, var,  const_string, var_name,  integer, dflt)
   }
 }
 
+/* FIXME -- some (most?) of this can/should be moved to the Pascal/WEB side. */
+#if defined(TeX) || defined(MP) || defined(MF)
+static void
+checkpoolpointer (poolpointer poolptr, size_t len)
+{
+  if (poolptr + len >= poolsize) {
+    fprintf (stderr, "\nstring pool overflow [%i bytes]\n", 
+            (int)poolsize); /* fixme */
+    exit(1);
+  }
+}
+
+#if !defined(pdfTeX) && !defined(pdfeTeX)
+static int
+maketexstring(const_string s)
+{
+  size_t len;
+  assert (s != 0);
+  len = strlen(s);
+  checkpoolpointer (poolptr, len);
+  while (len-- > 0)
+    strpool[poolptr++] = *s++;
+  return (makestring());
+}
+#endif
+
+int
+makefullnamestring()
+{
+  extern string fullnameoffile;
+  return maketexstring(fullnameoffile);
+}
+#endif
+
+#if defined(TeX)
+int
+compare_paths P2C(const_string, p1, const_string, p2)
+{
+  int ret;
+  while (
+#ifdef MONOCASE_FILENAMES
+                (((ret = (toupper(*p1) - toupper(*p2))) == 0) && (*p2 != 0))
+#else
+         (((ret = (*p1 - *p2)) == 0) && (*p2 != 0))
+#endif
+                || (IS_DIR_SEP(*p1) && IS_DIR_SEP(*p2))) {
+       p1++, p2++;
+  }
+  ret = (ret < 0 ? -1 : (ret > 0 ? 1 : 0));
+  return ret;
+}
+
+string
+gettexstring P1C(strnumber, s)
+{
+  size_t i, len;
+  string name;
+#ifndef Omega
+  len = strstart[s + 1] - strstart[s];
+#else
+  len = strstartar[s + 1 - 65535L] - strstartar[s - 65535L];
+#endif
+  name = (string)xmalloc (len + 1);
+#ifndef Omega
+  strncpy (name, (string)&strpool[strstart[s]], len);
+#else
+  /* Don't use strncpy.  The strpool is not made up of chars. */
+  for (i=0; i<len; i++) name[i] =  strpool[i+strstartar[s - 65535L]];
+#endif
+  name[len] = 0;
+  return name;
+}
+
+boolean
+isnewsource P2C(strnumber, srcfilename, int, lineno)
+{
+  size_t len;
+  char *name = gettexstring(srcfilename);
+  return (compare_paths(name, last_source_name) != 0 || lineno != last_lineno);
+}
+
+void
+remembersourceinfo P2C(strnumber, srcfilename,
+                                          int, lineno)
+{
+  size_t len;
+
+  if (last_source_name)
+       free(last_source_name);
+  last_source_name = gettexstring(srcfilename);
+  last_lineno = lineno;
+}
+
+poolpointer
+makesrcspecial P2C(strnumber, srcfilename,
+                                  int, lineno)
+{
+  poolpointer oldpoolptr = poolptr;
+  char *filename = gettexstring(srcfilename);
+  char buf[40];
+  size_t len = strlen(filename);
+  char * s = buf;  
+
+  sprintf (buf, "src:%d%s", lineno, 
+                  (isdigit(*filename) ? " " : ""));
+
+  if (poolptr + strlen(buf) + strlen(filename) >= poolsize) {
+       fprintf (stderr, "\nstring pool overflow\n"); /* fixme */
+       exit (1);
+  }
+  s = buf;
+  while (*s)
+    strpool[poolptr++] = *s++;
+
+  s = filename;
+  while (*s)
+    strpool[poolptr++] = *s++;
+       
+  return (oldpoolptr);
+}
+#endif
+
 #ifdef MP
 /* Invoke makempx (or troffmpx) to make sure there is an up-to-date
    .mpx file for a given .mp file.  (Original from John Hobby 3/14/90)  */
@@ -1263,8 +1594,15 @@ callmakempx P2C(string, mpname,  string, mpxname)
     if (!cnf_cmd)
       cnf_cmd = xstrdup (MPXCOMMAND);
 
-    cmd = concatn (cnf_cmd, troffmode ? " -troff " : " ",
+    if (troffmode)
+      cmd = concatn (cnf_cmd, " -troff ",
+                     mpname, " ", mpxname, NULL);
+    else if (mpost_tex_program && *mpost_tex_program)
+      cmd = concatn (cnf_cmd, " -tex=", mpost_tex_program, " ",
                    mpname, " ", mpxname, NULL);
+    else
+      cmd = concatn (cnf_cmd, " -tex ", mpname, " ", mpxname, NULL);
+
     /* Run it.  */
     ret = system (cmd);
     free (cmd);
@@ -1278,7 +1616,7 @@ callmakempx P2C(string, mpname,  string, mpxname)
 /* Metafont/MetaPost fraction routines. Replaced either by assembler or C.
    The assembler syntax doesn't work on Solaris/x86.  */
 #ifndef TeX
-#ifdef __sun__
+#if defined (__sun__) || defined (__cplusplus)
 #define NO_MF_ASM
 #endif
 #if defined(WIN32) && !defined(NO_MF_ASM)
@@ -1347,8 +1685,7 @@ connection with the use or performance of this software.
 #define TWEXP_28 (1.0/268435456.0)
 
 integer
-ztakefraction(p,q)		/* Approximate p*q/2^28 */
-integer p,q;
+ztakefraction P2C(integer, p, integer, q)     /* Approximate p*q/2^28 */
 {	register double d;
 	register integer i;
 	d = (double)p * (double)q * TWEXP_28;
@@ -1375,8 +1712,7 @@ integer p,q;
 }
 
 integer
-ztakescaled(p,q)		/* Approximate p*q/2^16 */
-integer p,q;
+ztakescaled P2C(integer, p, integer, q)		/* Approximate p*q/2^16 */
 {	register double d;
 	register integer i;
 	d = (double)p * (double)q * TWEXP_16;
@@ -1407,8 +1743,7 @@ integer p,q;
    and 8/1.  No pair of integers less than 2^31 has such a ratio.
 */
 integer
-zmakefraction(p,q)		/* Approximate 2^28*p/q */
-integer p,q;
+zmakefraction P2C(integer, p, integer, q)	/* Approximate 2^28*p/q */
 {	register double d;
 	register integer i;
 #ifdef DEBUG
@@ -1436,8 +1771,7 @@ integer p,q;
    and 2^15/1.  No pair of integers less than 2^31 has such a ratio.
 */
 integer
-zmakescaled(p,q)		/* Approximate 2^16*p/q */
-integer p,q;
+zmakescaled P2C(integer, p, integer, q)		/* Approximate 2^16*p/q */
 {	register double d;
 	register integer i;
 #ifdef DEBUG
@@ -1470,7 +1804,7 @@ integer p,q;
    graphics routines appropriate to the user's terminal.  stdout must be
    connected to a terminal for us to do any graphics.  */
 
-#ifdef MF_NAKED
+#ifdef MFNOWIN
 #undef AMIGAWIN
 #undef EPSFWIN
 #undef HP2627WIN
@@ -1643,7 +1977,7 @@ boolean
 initscreen P1H(void)
 {
   /* If MFTERM is set, use it.  */
-  char *tty_type = kpse_var_value ("MFTERM");
+  const_string tty_type = kpse_var_value ("MFTERM");
   
   if (tty_type == NULL)
     { 
@@ -1686,9 +2020,13 @@ initscreen P1H(void)
      automatically. Too frustrating for everyone involved.  */
   if (STREQ (tty_type, "xterm")) {
     fputs ("\nmf: Window support for X was not compiled into this binary.\n",
-            stderr);
-  fputs ("mf: To do so, rerun configure --with-x, recompile, and reinstall.\n",
            stderr);
+    fputs ("mf: There may be a binary called `mfw' on your system which\n",
+           stderr);
+    fputs ("mf: does contain X window support.\n\n", stderr);
+    fputs ("mf: If you need to recompile, remember to give the --with-x\n",
+           stderr);
+    fputs ("mf: option to configure\n\n", stderr);
     fputs ("mf: (Or perhaps you just failed to specify the mode.)\n", stderr);
   }
 
@@ -1713,9 +2051,8 @@ updatescreen P1H(void)
    the background color.  */
 
 void
-blankrectangle (left, right, top, bottom)
-    screencol left, right;
-    screenrow top, bottom;
+blankrectangle P4C(screencol, left, screencol, right,
+                   screenrow, top, screenrow, bottom)
 {
   if (mfwp->mfwsw_blankrect)
     (*mfwp->mfwsw_blankrect) (left, right, top, bottom);
@@ -1727,11 +2064,8 @@ blankrectangle (left, right, top, bottom)
    switch colors.  This goes on for VECTOR_SIZE transitions.  */
 
 void
-paintrow (row, init_color, transition_vector, vector_size)
-    screenrow row;
-    pixelcolor init_color;
-    transspec transition_vector;
-    screencol vector_size;
+paintrow P4C(screenrow, row, pixelcolor, init_color,
+             transspec, transition_vector, screencol, vector_size)
 {
   if (mfwp->mfwsw_paintrow)
     (*mfwp->mfwsw_paintrow) (row, init_color, transition_vector, vector_size);
