@@ -1,11 +1,26 @@
-/*
- * This file is part of the Omega project, which
- * is based in the web2c distribution of TeX.
- *
- * Copyright (c) 1994--1998 John Plaice and Yannis Haralambous
- */
+/* outocp.c: ASCII output for OCP files, mainly for debugging purposes.
+
+This file is part of Omega,
+which is based on the web2c distribution of TeX,
+
+Copyright (c) 1994--2001 John Plaice and Yannis Haralambous
+
+Omega is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+Omega is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with Omega; if not, write to the Free Software Foundation, Inc.,
+59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+
+*/
 
-#include <kpathsea/c-auto.h>
 #include <kpathsea/config.h>
 #include <kpathsea/types.h>
 #include <kpathsea/c-proto.h>
@@ -170,8 +185,7 @@ ctp_explain P1C(FILE *, input_file) {
            no_words_read, no_words_read);
 }
 
-int 
-main P2C(int, argc, string *, argv) {
+int main P2C(int, argc, string *, argv) {
 string input_name, full_input_name;
 FILE *input_file;
 
