@@ -1,6 +1,6 @@
 /*========================================================================*\
 
-Copyright (c) 1990-1999  Paul Vojta
+Copyright (c) 1990-2001  Paul Vojta
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -12,12 +12,13 @@ furnished to do so, subject to the following conditions:
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
 
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
-PAUL VOJTA BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
-IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
-CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
+IN NO EVENT SHALL PAUL VOJTA OR ANY OTHER AUTHOR OF THIS SOFTWARE BE
+LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 NOTE:
 	xdvi is based on prior work, as noted in the modification history
@@ -36,10 +37,31 @@ NOTE:
  *	The C environment	*
  *******************************/
 
-#include "c-auto.h"
+#ifdef __hpux
+/* On HP-UX 10.10 B and 20.10, compiling with _XOPEN_SOURCE + ..._EXTENDED
+ * leads to poll() not realizing that a file descriptor is writable in psgs.c.
+ */
+#define	_HPUX_SOURCE	1
+#else
+#define	_XOPEN_SOURCE	600
+#define	_XOPEN_SOURCE_EXTENDED	1
+#define	__EXTENSIONS__	1	/* needed to get struct timeval on SunOS 5.5 */
+#define	_SVID_SOURCE	1	/* needed to get S_IFLNK in glibc */
+#define	_BSD_SOURCE	1	/* needed to get F_SETOWN in glibc-2.1.3 */
+#endif
 
-#ifdef __linux
-#define FLAKY_SIGPOLL 1
+#include "c-auto.h"
+#include <errno.h>		/* FIXME: Check for availability? */
+
+/* Some O/S dependent kludges. */
+#ifdef _AIX
+#define _ALL_SOURCE 1
+#endif
+
+#if 0
+// FIXME: not in xdvi #ifdef __linux		/* needed only for kernel versions 2.1.xxx -- 2.2.8 */
+// FIXME: not in xdvi #define FLAKY_SIGPOLL 1
+// FIXME: not in xdvi #endif
 #endif
 
 #if STDC_HEADERS
@@ -62,8 +84,8 @@ NOTE:
 #define xfopen xdvi_xfopen
 
 /* For wchar_t et al., that the X files might want. */
-#include <kpathsea/systypes.h>
-#include <kpathsea/c-memstr.h>
+#include "kpathsea/systypes.h"
+#include "kpathsea/c-memstr.h"
 
 #include <X11/Xlib.h>	/* include Xfuncs.h, if available */
 #include <X11/Xutil.h>	/* needed for XDestroyImage */
@@ -76,15 +98,20 @@ NOTE:
 
 #ifndef	NOTOOL
 
+#define	TOOLKIT	1
+
 #include <X11/Intrinsic.h>
 #if	(defined(VMS) && (XtSpecificationRelease <= 4)) || defined(lint)
 #include <X11/IntrinsicP.h>
 #endif
-#define	TOOLKIT	1
+#if MOTIF
+#include <Xm/Xm.h>
+#endif
 
 #else	/* NOTOOL */
 
 #define	XtNumber(arr)	(sizeof(arr)/sizeof(arr[0]))
+#define	XtWindow(win)	(win)
 typedef	unsigned long	Pixel;
 typedef	char		Boolean;
 typedef	unsigned int	Dimension;
@@ -147,6 +174,9 @@ typedef	char		Bool3;		/* Yes/No/Maybe */
 # endif
 #endif
 
+#undef NeedFunctionPrototypes
+#define NeedFunctionPrototypes 0
+
 /* These are for X headers */
 #ifndef	NeedFunctionPrototypes
 #if	__STDC__
@@ -156,16 +186,7 @@ typedef	char		Bool3;		/* Yes/No/Maybe */
 #endif	/* STDC */
 #endif	/* NeedFunctionPrototypes */
 
-#ifndef HAVE_PROTOTYPES
-#if	__STDC__
-#define HAVE_PROTOTYPES 1
-#else   /* STDC */
-#define HAVE_PROTOTYPES 0
-#endif  /* STDC */
-#endif
-
-/* These are for xdvi */
-#if	HAVE_PROTOTYPES
+#if	NeedFunctionPrototypes
 #define	ARGS(x)	x
 #else
 #define	ARGS(x)	()
@@ -178,21 +199,51 @@ typedef	char		Bool3;		/* Yes/No/Maybe */
    definitions.  See <ansidecl.h> from the GNU C library.  P1H(void)
    also works for definitions of routines which take no args.  */
 
-#if __STDC__
+#if NeedFunctionPrototypes	/* Don't use __STDC__ here (gcc + SunOS 4) */
 
 #define	P1H(p1) (p1)
+#define P2H(p1,p2) (p1, p2)
+#define P3H(p1,p2,p3) (p1, p2, p3)
+#define P4H(p1,p2,p3,p4) (p1, p2, p3, p4)
+#define P5H(p1,p2,p3,p4,p5) (p1, p2, p3, p4, p5)
+#define P6H(p1,p2,p3,p4,p5,p6) (p1, p2, p3, p4, p5, p6)
+#define P7H(p1,p2,p3,p4,p5,p6,p7) (p1, p2, p3, p4, p5, p6,p7)
 
 #define	P1C(t1,n1)(t1 n1)
-#define	P2C(t1,n1, t2,n2)(t1 n1, t2 n2)
+#define	P2C(t1,n1, t2,n2) (t1 n1, t2 n2)
+#define P3C(t1,n1, t2,n2, t3,n3)(t1 n1, t2 n2, t3 n3)
+#define P4C(t1,n1, t2,n2, t3,n3, t4,n4) (t1 n1, t2 n2, t3 n3, t4 n4)
+#define P5C(t1,n1, t2,n2, t3,n3, t4,n4, t5,n5) \
+  (t1 n1, t2 n2, t3 n3, t4 n4, t5 n5)
+#define P6C(t1,n1, t2,n2, t3,n3, t4,n4, t5,n5, t6,n6) \
+  (t1 n1, t2 n2, t3 n3, t4 n4, t5 n5, t6 n6)
+#define P7C(t1,n1, t2,n2, t3,n3, t4,n4, t5,n5, t6,n6, t7,n7) \
+  (t1 n1, t2 n2, t3 n3, t4 n4, t5 n5, t6 n6, t7,n7)
 
-#else /* not __STDC__ */
+#else /* not NeedFunctionPrototypes */
 
 #define	P1H(p1) ()
+#define P2H(p1, p2) ()
+#define P3H(p1, p2, p3) ()
+#define P4H(p1,p2,p3,p4) ()
+#define P5H(p1, p2, p3, p4, p5) ()
+#define P6H(p1, p2, p3, p4, p5, p6) ()
+#define P6H(p1, p2, p3, p4, p5, p6, p7) ()
 
 #define	P1C(t1,n1) (n1) t1 n1;
 #define	P2C(t1,n1, t2,n2) (n1,n2) t1 n1; t2 n2;
+#define P3C(t1,n1, t2,n2, t3,n3) (n1,n2,n3) t1 n1; t2 n2; t3 n3;
+#define P4C(t1,n1, t2,n2, t3,n3, t4,n4) (n1,n2,n3,n4) \
+  t1 n1; t2 n2; t3 n3; t4 n4;
+#define P5C(t1,n1, t2,n2, t3,n3, t4,n4, t5,n5) (n1,n2,n3,n4,n5) \
+  t1 n1; t2 n2; t3 n3; t4 n4; t5 n5;
+#define P6C(t1,n1, t2,n2, t3,n3, t4,n4, t5,n5, t6,n6) (n1,n2,n3,n4,n5,n6) \
+  t1 n1; t2 n2; t3 n3; t4 n4; t5 n5; t6 n6;
+#define P7C(t1,n1, t2,n2, t3,n3, t4,n4, t5,n5, t6,n6, t7,n7) \
+  (n1,n2,n3,n4,n5,n6,n7) \
+  t1 n1; t2 n2; t3 n3; t4 n4; t5 n5; t6 n6; t7,n7;
 
-#endif /* not __STDC__ */
+#endif /* not NeedFunctionPrototypes */
 
 #endif /* not KPATHSEA */
 
@@ -200,13 +251,19 @@ typedef	char		Bool3;		/* Yes/No/Maybe */
 #define	NeedWidePrototypes	NeedFunctionPrototypes
 #endif
 
-#include <kpathsea/c-vararg.h>
+#ifndef	NeedVarargsPrototypes
+#define	NeedVarargsPrototypes	NeedFunctionPrototypes
+#endif
+
+#include "kpathsea/c-vararg.h"
 
 #ifndef	_XFUNCPROTOBEGIN
 #define	_XFUNCPROTOBEGIN
 #define	_XFUNCPROTOEND
 #endif
 
+#include "xdvi-config.h"
+
 #ifndef	_Xconst
 #if	__STDC__
 #define	_Xconst	const
@@ -224,14 +281,22 @@ typedef	char		Bool3;		/* Yes/No/Maybe */
 #endif
 
 #ifndef	NORETURN
-#ifdef	__GNUC__
-#define	NORETURN	volatile
-#else
-#define	NORETURN	/* nothing */
-#endif
+#  ifdef	__GNUC__
+#    ifndef __STRICT_ANSI__
+#      define	NORETURN	volatile
+#    else
+#      define	NORETURN
+#    endif
+#  else
+#    define	NORETURN	/* nothing */
+#  endif
 #endif
 
 #ifndef	OPEN_MODE
+/*
+ * SU, 2001/01/07: xdvi defines OPEN_MODE as "r" or as "r", "ctx=stm" (for VMS),
+ * but we use the definition of FOPEN_R_MODE from kpathsea/c-fopen.h instead:
+ */
 #define OPEN_MODE FOPEN_R_MODE
 #endif	/* OPEN_MODE */
 
@@ -255,9 +320,12 @@ typedef	char		Bool3;		/* Yes/No/Maybe */
 #define	Strcat	(void) strcat
 #define	Strcpy	(void) strcpy
 
-#ifdef HTEX
-char *urlocalize P1H(char *filename);
-extern int lastwwwopen;
+#ifndef __LINE__
+#define __LINE__ 0
+#endif
+
+#ifndef __FILE__
+#define __FILE__ "?"
 #endif
 
 /********************************
@@ -283,7 +351,7 @@ typedef	Boolean		wide_bool;
 #define	WIDENINT
 #endif
 
-#if defined(MAKETEXPK) && !defined(MKTEXPK)
+#if defined(MAKEPK) && !defined(MKTEXPK)
 #define	MKTEXPK 1
 #endif
 
@@ -307,7 +375,17 @@ typedef	Boolean		wide_bool;
 
 extern	BMUNIT	bit_masks[BMBITS + 1];
 
+/*
+ * added SU:
+ * when magnifier is active, mane.shrinkfactor is still the shrinkfactor of the
+ * main window, while shrink_factor is the shrinkfactor of the magnifier.
+ */
+#define MAGNIFIER_ACTIVE (mane.shrinkfactor != shrink_factor)
+
 struct frame {
+  	/* dvi_h and dvi_v is the horizontal and vertical baseline position;
+	   it is the responsability of the set_char procedure to update
+	   them. */
 	struct framedata {
 		long dvi_h, dvi_v, w, x, y, z;
 		int pxl_v;
@@ -337,6 +415,10 @@ struct drawinf {	/* this information is saved when using virtual fonts */
 
 EXTERN	struct drawinf	currinf;
 
+/* Points to drawinf record containing current dvi file location (for update by
+   geom_scan).  */
+EXTERN	struct drawinf	*dvi_pointer_frame	INIT(NULL);
+
 /* entries below with the characters 'dvi' in them are actually stored in
    scaled pixel units */
 
@@ -356,12 +438,16 @@ EXTERN	int	pageno_correct	INIT(1);
 EXTERN	long	magnification;
 EXTERN	double	dimconv;
 EXTERN	double	tpic_conv;
-EXTERN	int	n_files_left;	/* for LRU closing of fonts */
-EXTERN	time_t	dvi_time;		/* last mod. time for dvi file */
+EXTERN	int	n_files_left	INIT(32767);	/* for LRU closing of fonts */
 EXTERN	unsigned int	page_w, page_h;
 
+#ifndef	BDPI
+#define	BDPI	600
+#endif
+
+
 #if	defined(GS_PATH) && !defined(PS_GS)
-#define	PS_GS
+#define	PS_GS	1
 #endif
 
 #if	defined(PS_DPS) || defined(PS_NEWS) || defined(PS_GS)
@@ -392,6 +478,11 @@ EXTERN	Boolean	warn_spec_now;
  */
 EXTERN	Boolean	psfig_begun	INIT(False);
 
+/*
+ * Page on which to draw box from forward source special searching.
+ */
+EXTERN	int	source_fwd_box_page	INIT(-1);	/* -1 means no box */
+
 
 /*
  * Bitmap structure for raster ops.
@@ -449,35 +540,39 @@ struct macro {
 
 #define	NOMAGSTP (-29999)
 
-typedef	void (*read_char_proc) ARGS((register struct font *, wide_ubyte));
+typedef	void (*read_char_proc) ARGS((struct font *, wide_ubyte));
 
 struct font {
-	struct font *next;		/* link to next font info block */
-	char *fontname;			/* name of font */
-	float fsize;			/* size information (dots per inch) */
-	int magstepval;			/* magstep number * two, or NOMAGSTP */
-	FILE *file;			/* open font file or NULL */
-	char *filename;			/* name of font file */
-	long checksum;			/* checksum */
-	unsigned short timestamp;	/* for LRU management of fonts */
-	ubyte flags;			/* flags byte (see values below) */
+  struct font *next;		/* link to next font info block */
+  char *fontname;		/* name of font */
+  float fsize;			/* size information (dots per inch) */
+  int magstepval;		/* magstep number * two, or NOMAGSTP */
+  FILE *file;			/* open font file or NULL */
+  char *filename;		/* name of font file */
+  long checksum;		/* checksum */
+  unsigned short timestamp;	/* for LRU management of fonts */
+  ubyte flags;			/* flags byte (see values below) */
 #ifdef Omega
-	wide_ubyte maxchar;			/* largest character code */
+  wide_ubyte maxchar;		/* largest character code */
 #else
-	ubyte maxchar;			/* largest character code */
+  ubyte maxchar;		/* largest character code */
 #endif
-	double dimconv;			/* size conversion factor */
-	set_char_proc set_char_p;	/* proc used to set char */
-		/* these fields are used by (loaded) raster fonts */
-	read_char_proc read_char;	/* function to read bitmap */
-	struct glyph *glyph;
-		/* these fields are used by (loaded) virtual fonts */
-	struct font **vf_table;		/* list of fonts used by this vf */
-	struct tn *vf_chain;		/* ditto, if TeXnumber >= VFTABLELEN */
-	struct font *first_font;	/* first font defined */
-	struct macro *macro;
-		/* I suppose the above could be put into a union, but we */
-		/* wouldn't save all that much space. */
+  double dimconv;		/* size conversion factor */
+  set_char_proc set_char_p;	/* proc used to set char */
+  /* these fields are used by (loaded) raster fonts */
+  read_char_proc read_char;	/* function to read bitmap */
+  struct glyph *glyph;
+  /* these fields are used by (loaded) virtual fonts */
+  struct font **vf_table;	/* list of fonts used by this vf */
+  struct tn *vf_chain;		/* ditto, if TeXnumber >= VFTABLELEN */
+  struct font *first_font;	/* first font defined */
+  struct macro *macro;
+  /* I suppose the above could be put into a union, but we */
+  /* wouldn't save all that much space. */
+
+  /* These were added for t1 use */
+  int t1id;
+  long scale;
 };
 
 #define	FONT_IN_USE	1	/* used for housekeeping */
@@ -508,93 +603,123 @@ EXTERN	unsigned short	current_timestamp INIT(0);
  */
 
 extern	struct _resource {
-#if CFGFILE && TOOLKIT
-	_Xconst char	*progname;
+#if TOOLKIT && CFGFILE
+		_Xconst char	*progname;
 #endif
+		Boolean		regression;
 #if TOOLKIT
-	int		shrinkfactor;
+		int		shrinkfactor;
+		_Xconst char	*main_translations;
+		_Xconst char	*wheel_translations;
 #endif
-	int		_density;
+		int		wheel_unit;
+		int		_density;
 #ifdef	GREY
-	float		_gamma;
+		float		_gamma;
 #endif
-	int		_pixels_per_inch;
-	Boolean		_delay_rulers;
-	int		_tick_length;
-	char		*_tick_units;
-	_Xconst	char	*sidemargin;
-	_Xconst	char	*topmargin;
-	_Xconst	char	*xoffset;
-	_Xconst	char	*yoffset;
-	_Xconst	char	*paper;
-	_Xconst	char	*_alt_font;
-	Boolean		makepk;
-	_Xconst	char	*mfmode;
-	Boolean		_list_fonts;
-	Boolean		reverse;
-	Boolean		_warn_spec;
-	Boolean		_hush_chars;
-	Boolean		_hush_chk;
-	Boolean		safer;
+		int		_pixels_per_inch;
+		Boolean		_delay_rulers;
+		int		_tick_length;
+		char		*_tick_units;
+		_Xconst	char	*sidemargin;
+		_Xconst	char	*topmargin;
+		_Xconst	char	*xoffset;
+		_Xconst	char	*yoffset;
+		_Xconst	char	*paper;
+		_Xconst	char	*_alt_font;
+#ifdef MKTEXPK
+		Boolean		makepk;
+#endif
+		_Xconst	char	*mfmode;
+	    _Xconst char	*editor;
+                Boolean		t1lib;
+	    _Xconst char	*src_pos;
+		Boolean		_list_fonts;
+		Boolean		reverse;
+		Boolean		_warn_spec;
+		Boolean		_hush_chars;
+		Boolean		_hush_chk;
+		Boolean		_hush_stdout;
+		Boolean		safer;
 #if defined(VMS) || !defined(TOOLKIT)
-	_Xconst	char	*fore_color;
-	_Xconst	char	*back_color;
+		_Xconst	char	*fore_color;
+		_Xconst	char	*back_color;
 #endif
-	Pixel		_fore_Pixel;
-	Pixel		_back_Pixel;
+		Pixel		_fore_Pixel;
+		Pixel		_back_Pixel;
 #ifdef TOOLKIT
-	Pixel		_brdr_Pixel;
-	Pixel		_hl_Pixel;
-	Pixel		_cr_Pixel;
+		Pixel		_brdr_Pixel;
+		Pixel		_hl_Pixel;
+		Pixel		_cr_Pixel;
 #endif
-	_Xconst	char	*icon_geometry;
-	Boolean		keep_flag;
-	Boolean		copy;
-	Boolean		thorough;
+		_Xconst	char	*icon_geometry;
+		Boolean		keep_flag;
+		Boolean		copy;
+		Boolean		thorough;
 #if	PS
-	/* default is to use DPS, then NEWS, then GhostScript;
-	 * we will figure out later on which one we will use */
-	Boolean		_postscript;
-	Boolean		prescan;
-	Boolean		allow_shell;
+		/* default is to use DPS, then NEWS, then GhostScript;
+		 * we will figure out later on which one we will use */
+		Boolean		_postscript;
+		Boolean		prescan;
+		Boolean		allow_shell;
 #ifdef	PS_DPS
-	Boolean		useDPS;
+		Boolean		useDPS;
 #endif
 #ifdef	PS_NEWS
-	Boolean		useNeWS;
+		Boolean		useNeWS;
 #endif
 #ifdef	PS_GS
-	Boolean		useGS;
-	Boolean		gs_safer;
-	Boolean		gs_alpha;
-	_Xconst	char	*gs_path;
-	_Xconst	char	*gs_palette;
+		Boolean		useGS;
+		Boolean		gs_safer;
+		Boolean		gs_alpha;
+		_Xconst	char	*gs_path;
+		_Xconst	char	*gs_palette;
 #endif
 #endif	/* PS */
-	_Xconst	char	*debug_arg;
-	Boolean		version_flag;
+		_Xconst	char	*debug_arg;
+		Boolean		version_flag;
 #ifdef	BUTTONS
-	Boolean		expert;
-	int		shrinkbutton[4];
+		Boolean		expert;
+		Boolean		statusline;
+		_Xconst char	*button_translations;
+		int		shrinkbutton[9];
+		int		btn_side_spacing;
+		int		btn_top_spacing;
+		int		btn_between_spacing;
+		int		btn_between_extra;
+		int		btn_border_width;
 #endif
-	_Xconst	char	*mg_arg[5];
+		_Xconst	char	*mg_arg[5];
 #ifdef	GREY
-	Boolean		_use_grey;
-	Bool3		install;
+		Boolean		_use_grey;
+		Bool3		install;
 #endif
 #ifdef GRID
-	int _grid_mode;
-	char *grid1_color, *grid2_color, *grid3_color;
+		int _grid_mode;
+		char *grid1_color, *grid2_color, *grid3_color;
 #ifdef	TOOLKIT
-	Pixel	_grid1_Pixel, _grid2_Pixel, _grid3_Pixel;
+		Pixel	_grid1_Pixel, _grid2_Pixel, _grid3_Pixel;
 #endif /* TOOLKIT */
 #endif /* GRID */
 #ifdef HTEX
-	Boolean	_underline_link;
-	char	*_browser;
-	char	*_URLbase;
-	char	*_scroll_pages;
+		Boolean	_underline_link;
+		char	*_browser;
+		char	*_URLbase;
+		char	*_scroll_pages;
 #endif /* HTEX */
+		char * _help_topics_button_label;
+		char * _help_quit_button_label;
+		char * _help_intro;
+		char * _help_general_menulabel;
+		char * _help_general;
+		char * _help_hypertex_menulabel;
+		char * _help_hypertex;
+		char * _help_othercommands_menulabel;
+		char * _help_othercommands;
+		char * _help_pagemotion_menulabel;
+		char * _help_pagemotion;
+		char * _help_sourcespecials_menulabel;
+		char * _help_sourcespecials;
 } resource;
 
 /* As a convenience, we define the field names without leading underscores
@@ -608,6 +733,7 @@ extern	struct _resource {
 #define	warn_spec	resource._warn_spec
 #define	hush_chars	resource._hush_chars
 #define	hush_chk	resource._hush_chk
+#define	hush_stdout	resource._hush_stdout
 #ifdef  GREY
 #define	use_grey	resource._use_grey
 #endif
@@ -620,7 +746,8 @@ extern	struct _resource {
 #define URLbase		resource._URLbase
 #define scroll_pages	resource._scroll_pages
 #define KPSE_DEBUG_HYPER	6
-#endif
+#endif /* HTEX */
+
 
 #ifndef TOOLKIT
 EXTERN	Pixel		brdr_Pixel;
@@ -646,24 +773,69 @@ extern	struct	mg_size_rec {
 
 EXTERN	int		debug		INIT(0);
 
-#define	DBG_BITMAP	1
-#define	DBG_DVI		2
-#define	DBG_PK		4
-#define	DBG_BATCH	8
-#define	DBG_EVENT	16
-#define	DBG_OPEN	32
-#define	DBG_PS		64
-#define	DBG_STAT	128
-#define	DBG_HASH	256
-#define	DBG_PATHS	512
-#define	DBG_EXPAND	1024
-#define	DBG_SEARCH	2048
-#ifdef HTEX
-#define	DBG_HYPER	4096
-#define	DBG_ANCHOR	8192
+/* for temporary debugging statements */
+#ifdef MYDEBUG
+#define MYTRACE(X)   do {                                         \
+                fprintf(stderr, "%s:%d: ", __FILE__, __LINE__);   \
+                fprintf X;                                        \
+                fprintf(stderr, "\n");                            \
+     } while(0)
+#else
+#define MYTRACE(X)
 #endif
-#define	DBG_ALL		(~DBG_BATCH)
 
+#define	DBG_BITMAP		1
+#define	DBG_DVI			2
+#define	DBG_PK			4
+#define	DBG_BATCH		8
+#define	DBG_EVENT		16
+#define	DBG_OPEN		32
+#define	DBG_PS			64
+#define	DBG_STAT		128
+#define	DBG_HASH		256
+#define	DBG_PATHS		512
+#define	DBG_EXPAND		1024
+#define	DBG_SEARCH		2048
+/* SU 2000/03/10: making this optional doesn't work with NOTOOL */
+/* #ifdef HTEX */
+#define	DBG_HYPER		4096
+#define	DBG_ANCHOR		8192
+/* #endif */
+#define DBG_SRC_SPECIALS	16384
+#define DBG_CLIENT		32768
+#define DBG_T1			65536
+#define	DBG_ALL			(~DBG_BATCH)
+
+#ifdef NDEBUG
+#define TRACE_SRC(X)	/* as nothing */
+#define TRACE_CLIENT(X)	/* as nothing */
+#define TRACE_T1(X)	/* as nothing */
+#else
+#define TRACE_SRC(X)								\
+    do {									\
+		if (debug & DBG_SRC_SPECIALS) {					\
+    		fprintf(stderr, "%s:%d: SRC: ", __FILE__, __LINE__);		\
+    		fprintf X;							\
+    		fprintf(stderr, "\n");						\
+    	}									\
+     } while(0)
+#define TRACE_CLIENT(X)								\
+    do {									\
+		if (debug & DBG_CLIENT) {					\
+    		fprintf(stderr, "%s:%d: CLIENT: ", __FILE__, __LINE__);		\
+    		fprintf X;							\
+    		fprintf(stderr, "\n");						\
+    	}									\
+     } while(0)
+#define TRACE_T1(X)								\
+    do {									\
+		if (debug & DBG_T1) {						\
+    		fprintf(stderr, "%s:%d: T1: ", __FILE__, __LINE__);		\
+    		fprintf X;							\
+    		fprintf(stderr, "\n");						\
+    	}									\
+     } while(0)
+#endif
 
 EXTERN	int		offset_x, offset_y;
 EXTERN	unsigned int	unshrunk_paper_w, unshrunk_paper_h;
@@ -672,18 +844,11 @@ EXTERN	unsigned int	unshrunk_page_w, unshrunk_page_h;
 EXTERN  unsigned int	unshrunk_paper_unit;
 #endif /* GRID */
 
-#ifdef HTEX
-EXTERN	int	HTeXnext_extern; /* Bring up next file in new xdvi window */
-EXTERN	char	*temporary_dir	INIT(NULL);
-EXTERN	char	*anchor_name	INIT(NULL);
-EXTERN	Boolean	URL_aware	INIT(False);
-EXTERN	Boolean	highlight	INIT(False);
-extern int HTeXAnestlevel;	/* Hypertext nesting level */
-extern int HTeXreflevel;	/* flag for whether we are inside an href */
-#endif
-
-EXTERN	char		*dvi_name	INIT(NULL);
+EXTERN	char		*dvi_name	INIT(NULL);	/* dvi file name */
 EXTERN	FILE		*dvi_file;		/* user's file */
+EXTERN	time_t		dvi_time;		/* last modification time */
+EXTERN	unsigned char	*dvi_property;		/* for setting in window */
+EXTERN	size_t		dvi_property_length;
 EXTERN	_Xconst char	*prog;
 EXTERN	int		bak_shrink;		/* last shrink factor != 1 */
 EXTERN	Dimension	window_w, window_h;
@@ -693,27 +858,49 @@ EXTERN	int		home_x, home_y;
 
 EXTERN	Display		*DISP;
 EXTERN	Screen		*SCRN;
+#if TOOLKIT
+extern	XtActionsRec	Actions[];
+extern	Cardinal	num_actions;
+#endif
+#ifndef NOTOOL
+EXTERN	XtAccelerators	accelerators;
+#endif
 #ifdef GREY
 EXTERN	int		screen_number;
 EXTERN	Visual		*our_visual;
 EXTERN	unsigned int	our_depth;
 EXTERN	Colormap	our_colormap;
-EXTERN	GC		copyGC;
 #else
 #define	our_depth	(unsigned int) DefaultDepthOfScreen(SCRN)
 #define	our_visual	DefaultVisualOfScreen(SCRN)
 #define	our_colormap	DefaultColormapOfScreen(SCRN)
-#define	copyGC		DefaultGCOfScreen(SCRN)
 #endif
 EXTERN	GC		ruleGC;
 EXTERN	GC		foreGC, highGC;
 EXTERN	GC		foreGC2;
+EXTERN	GC		copyGC;
 #ifdef GRID
 EXTERN  GC      grid1GC, grid2GC, grid3GC;
 #endif /* GRID */
 EXTERN	Boolean		copy;
 
-EXTERN	Cursor		redraw_cursor, ready_cursor;
+EXTERN	Cursor		redraw_cursor, ready_cursor, link_cursor, drag_cursor[3];
+
+#if MOTIF && BUTTONS
+EXTERN	XtTranslations	wheel_trans_table	INIT(NULL);
+#endif
+
+/* BEGIN CHUNK xdvi.h 6 */
+#include "message-window.h"
+
+#ifdef HAVE_ASSERT_H
+#include <assert.h>
+#else
+#define assert(expr) /* as nothing */
+#endif
+
+EXTERN char *global_dvi_name INIT(NULL); /* save filename argument from xdvi call, including the path  */
+/* END CHUNK xdvi.h 6 */
 
 #ifdef	GREY
 EXTERN	Pixel		*pixeltbl;
@@ -739,9 +926,12 @@ EXTERN	int		min_x, max_x, min_y, max_y;
 #define	shrink_factor	currwin.shrinkfactor
 
 #ifdef	TOOLKIT
-EXTERN	Widget		top_level, vport_widget, draw_widget, clip_widget;
-#ifdef HTEX
-EXTERN	Widget	pane_widget, anchor_search, anchor_info;
+EXTERN	Widget		top_level	INIT(0);
+EXTERN	Widget		vport_widget, draw_widget, clip_widget;
+#if 0
+// #ifdef HTEX
+// EXTERN	Widget	pane_widget, anchor_search, anchor_info;
+// #endif /* HTEX */
 #endif
 #ifdef MOTIF
 EXTERN	Widget		shrink_button[4];
@@ -750,13 +940,22 @@ EXTERN	Widget		x_bar, y_bar;	/* horizontal and vert. scroll bars */
 #ifdef	BUTTONS
 #ifndef MOTIF
 #define	XTRA_WID	79
-#else
+#ifdef STATUSLINE
+/* this is only for the initialization; the statusline will reset it to a more adequate value: */
+#define	XTRA_H	17 
+#endif /* STATUSLINE */
+#else /* MOTIF */
 #define	XTRA_WID	120
-#endif
-EXTERN	Widget	form_widget;
-#endif
+#ifdef STATUSLINE
+#define	XTRA_H	17 
+#endif /* STATUSLINE */
+#endif /* MOTIF */
+EXTERN	Widget	form_widget, paned;
+EXTERN	int		xtra_wid	INIT(0);
+extern	_Xconst char	default_button_config[]; /* defined in events.c */
+#endif /* BUTTONS */
 #else	/* !TOOLKIT */
-EXTERN	Window	top_level;
+EXTERN	Window	top_level	INIT(0);
 
 #define	BAR_WID		12	/* width of darkened area */
 #define	BAR_THICK	15	/* gross amount removed */
@@ -765,11 +964,36 @@ EXTERN	Window	top_level;
 EXTERN	jmp_buf		dvi_env;	/* mechanism to relay dvi file errors */
 EXTERN	_Xconst	char	*dvi_oops_msg;	/* error message */
 
-EXTERN	char	*ffline	INIT(NULL);	/* an array used by filefind to store */
-					/* the file name being formed.  */
-					/* It expands as needed. */
-					/* Also used elsewhere.  */
-EXTERN	int	ffline_len INIT(0);	/* current length of ffline[] */
+EXTERN	char	*ffline	INIT(NULL);	/* array used to store filenames */
+EXTERN	size_t	ffline_len INIT(0);	/* current length of ffline[] */
+
+/*
+ *	Used by the geometry-scanning routines.
+ *	It passes pointers to routines to be called at certain
+ *	points in the dvi file, and other information.
+ */
+
+struct geom_info {
+	void	(*geom_box)	ARGS((struct geom_info *,
+				  long, long, long, long));
+	void	(*geom_special)	ARGS((struct geom_info *, _Xconst char *));
+	jmp_buf	done_env;
+	void	*geom_data;
+};
+
+typedef	void	(*mouse_proc)	ARGS((XEvent *));
+extern	void	null_mouse	ARGS((XEvent *));
+
+EXTERN	mouse_proc	mouse_motion	INIT(null_mouse);
+EXTERN	mouse_proc	mouse_release	INIT(null_mouse);
+
+/* Used for source special lookup (forward search).  */
+
+EXTERN	Atom	atoms[3];
+
+#define	ATOM_XDVI_WINDOWS	(atoms[0])
+#define	ATOM_DVI_FILE		(atoms[1])
+#define	ATOM_SRC_GOTO		(atoms[2])
 
 #ifdef	SELFAUTO
 EXTERN	_Xconst	char	*argv0;		/* argv[0] */
@@ -806,36 +1030,64 @@ extern	struct psprocs	{
  *******************************/
 
 _XFUNCPROTOBEGIN
+
+extern void fork_process ARGS((_Xconst char *file, char *_Xconst argv[]));
+extern void print_child_error ARGS((void));
+
 #ifdef	BUTTONS
-extern	void	create_buttons ARGS((XtArgVal));
+extern	void	create_buttons ARGS((void));
+
+#ifdef STATUSLINE
+extern Widget statusline;
 #endif
+
+/* statusline stuff. The following function and #defines are also
+ * needed when compiling without statusline support.
+ */
+typedef enum STATUS_TIMER_
+{
+    STATUS_FOREVER = 0,
+    STATUS_SHORT = 10,
+    STATUS_MEDIUM = 40,
+    STATUS_LONG = 90
+} STATUS_TIMER;
+
+extern void print_statusline PVAR2H(STATUS_TIMER, _Xconst char *);
+
+#ifdef STATUSLINE
+extern void create_statusline ARGS((void));
+extern void handle_statusline_resize ARGS((void));
+#endif
+
+#if !MOTIF
+extern	void	set_button_panel_height ARGS((XtArgVal));
+#endif /* !MOTIF */
+#endif /* BUTTONS */
 #ifdef	GREY
 extern	void	init_plane_masks ARGS((void));
-#endif
-extern	void	init_colors ARGS((void));
+extern	void	init_pix ARGS((void));
+#endif /* GREY */
 extern	void	reconfig ARGS((void));
 #ifdef	TOOLKIT
-extern	void	handle_key ARGS((Widget, XtPointer, XEvent *, Boolean *));
 extern	void	handle_resize ARGS((Widget, XtPointer, XEvent *, Boolean *));
-extern	void	handle_button ARGS((Widget, XtPointer, XEvent *, Boolean *));
-extern	void	handle_motion ARGS((Widget, XtPointer, XEvent *, Boolean *));
-extern	void	handle_release ARGS((Widget, XtPointer, XEvent *, Boolean *));
-extern	void	handle_exp ARGS((Widget, XtPointer, XEvent *, Boolean *));
-#endif
+extern	void	handle_expose ARGS((Widget, XtPointer, XEvent *, Boolean *));
+extern  void	handle_property_change ARGS((Widget, XtPointer, XEvent *, Boolean *));
+#endif /* TOOLKIT */
 #ifdef MOTIF
 extern	void	file_pulldown_callback ARGS((Widget, XtPointer, XtPointer));
 extern	void	navigate_pulldown_callback ARGS((Widget, XtPointer, XtPointer));
 extern	void	scale_pulldown_callback ARGS((Widget, XtPointer, XtPointer));
 extern	void	set_shrink_factor ARGS((int));
-#endif
+#endif /* MOTIF */
 extern	void	showmessage ARGS((_Xconst char *));
 #if	PS
 extern	void	ps_read_events ARGS((wide_bool, wide_bool));
 #define	read_events(wait)	ps_read_events(wait, True)
-#else
+#else /* PS */
 extern	void	read_events ARGS((wide_bool));
-#endif
+#endif /* PS */
 extern	void	redraw_page ARGS((void));
+extern	void	enable_intr ARGS((void));
 extern	void	do_pages ARGS((void));
 extern	void	reset_fonts ARGS((void));
 extern	void	realloc_font ARGS((struct font *, wide_ubyte));
@@ -846,6 +1098,9 @@ extern	struct font	*define_font ARGS((FILE *, wide_ubyte,
 			struct tn **));
 extern	void	init_page ARGS((void));
 extern	void	open_dvi_file ARGS((void));
+extern	void	form_dvi_property ARGS((void));
+extern	void	init_dvi_file ARGS((void));
+extern	void	set_dvi_property ARGS((void));
 extern	Boolean	check_dvi_file ARGS((void));
 #ifdef GRID
 extern	void	put_grid ARGS((int, int, unsigned int, unsigned int, unsigned int, GC, GC, GC));
@@ -854,91 +1109,136 @@ extern	void	put_grid ARGS((int, int, unsigned int, unsigned int, unsigned int, G
 extern	long	set_char ARGS((wide_ubyte));
 extern	long	load_n_set_char ARGS((wide_ubyte));
 extern	long	set_vf_char ARGS((wide_ubyte));
-#else
+extern	long	set_t1_char P1H(wide_ubyte);
+#else /* TEXXET */
 extern	void	set_char ARGS((wide_ubyte, wide_ubyte));
 extern	void	load_n_set_char ARGS((wide_ubyte, wide_ubyte));
 extern	void	set_vf_char ARGS((wide_ubyte, wide_ubyte));
-#endif
+extern	void	set_t1_char ARGS((wide_ubyte, wide_ubyte));
+#endif /* TEXXET */
+
+#ifdef T1LIB
+extern	int	tfmload ARGS((char *, long *, long *));
+extern	void	read_T1_char ARGS((struct font *, wide_ubyte));
+extern  int	find_T1_font ARGS((char *));
+#endif /* T1LIB */
+extern  int	getpsinfo ARGS((char *));
 extern	void	draw_page ARGS((void));
+extern	void	source_reverse_search ARGS((int, int, wide_bool));
+extern	void	source_special_show ARGS((wide_bool));
+extern	void	source_forward_search ARGS((_Xconst char *));
+#ifdef T1LIB
+extern  void	init_t1 ARGS((void));
+#endif
 #if	CFGFILE
 #ifndef	CFG2RES
 extern	void	readconfig ARGS((void));
-#else
+#else /* CFG2RES */
 extern	void	readconfig ARGS((_Xconst struct cfg2res *,
 		  _Xconst struct cfg2res *, XtResource *, XtResource *));
 #endif	/* CFG2RES */
 #endif	/* CFGFILE */
 extern	void	init_font_open ARGS((void));
-extern	FILE	*font_open ARGS((_Xconst char *, char **, double, int *, int,
-			char **));
-#if	PS
-extern	void	ps_newdoc ARGS((void));
-extern	void	ps_destroy ARGS((void));
-#endif
-extern	void	applicationDoSpecial ARGS((char *));
-#if	PS
-extern	void	scan_special ARGS((char *));
-#endif
-extern	NORETURN void	oops ARGS((_Xconst char *message, ...));
-#ifndef KPATHSEA
-extern	void	*xmalloc ARGS((unsigned));
-extern	void	*xrealloc ARGS((void *, unsigned));
-extern	char	*xstrdup ARGS((_Xconst char *, int));
-extern	char	*xmemdump ARGS((_Xconst char *, int));
-#endif
-extern	void	expandline ARGS((int));
-extern	void	alloc_bitmap ARGS((struct bitmap *));
-#ifndef KPATHSEA
-extern	void	xputenv ARGS((_Xconst char *, _Xconst char *));
-#endif
-extern	int	memicmp ARGS((_Xconst char *, _Xconst char *, size_t));
-extern FILE    *xfopen P2H(char *, OPEN_MODE_ARGS);
-#ifdef HTEX
-extern FILE    *xfopen_local P2H(char *, OPEN_MODE_ARGS);
-#else
-#define xfopen_local	xfopen
-#endif
-extern	int	xpipe ARGS((int *));
-extern	DIR	*xdvi_xopendir ARGS((_Xconst char *));
-extern	_Xconst	struct passwd *ff_getpw ARGS((_Xconst char **, _Xconst char *));
-extern	unsigned long	num ARGS((FILE *, int));
-extern	long	snum ARGS((FILE *, int));
-extern	void	read_PK_index ARGS((struct font *, wide_bool));
-extern	void	read_GF_index ARGS((struct font *, wide_bool));
-#ifdef Omega
-extern	unsigned long read_VF_index ARGS((struct font *, wide_bool));
-#else
-extern	void	read_VF_index ARGS((struct font *, wide_bool));
-#endif
+#if T1 /* T1 only */
+extern	FILE	*font_open ARGS((char *, char **, double, int *, int, char **,int *));
+#else /* T1 */
+extern	FILE	*font_open ARGS((char *, char **, double, int *, int, char **));
+#endif /* T1 */
 
 #if	PS
+extern	void	ps_clear_cache ARGS((void));
+extern	void	ps_newdoc ARGS((void));
+extern	void	ps_destroy ARGS((void));
+extern	void	scan_special ARGS((char *));
 extern	void	drawbegin_none ARGS((int, int, _Xconst char *));
 extern	void	draw_bbox ARGS((void));
 extern	void	NullProc ARGS((void));
 #ifdef	PS_DPS
 extern	Boolean	initDPS ARGS((void));
-#endif
+#endif /* PS_DPS */
 #ifdef	PS_NEWS
 extern	Boolean	initNeWS ARGS((void));
-#endif
+#endif /* PS_NEWS */
 #ifdef	PS_GS
 extern	Boolean	initGS ARGS((void));
-#endif
-#endif	/* PS */
+#endif /* PS_GS */
+#endif /* PS */
+
+extern	void	applicationDoSpecial ARGS((char *));
+
+extern	NORETURN void	xdvi_exit ARGS((int));
+
+extern	void	geom_do_special ARGS((struct geom_info *, _Xconst char *, double));
+
+extern NORETURN void oops PVAR1H(_Xconst char *);
+
+#ifndef KPATHSEA
+extern	void	*xmalloc ARGS((unsigned));
+extern	void	*xrealloc ARGS((void *, unsigned));
+extern	char	*xstrdup ARGS((_Xconst char *));
+extern	char	*xmemdump ARGS((_Xconst char *, size_t));
+extern	void	xputenv ARGS((_Xconst char *, _Xconst char *));
+#endif /* KPATHSEA */
+extern	void	expandline ARGS((size_t));
+extern	void	alloc_bitmap ARGS((struct bitmap *));
+
+extern	int	memicmp ARGS((_Xconst char *, _Xconst char *, size_t));
+extern	FILE	*xfopen ARGS((_Xconst char *, OPEN_MODE_ARGS));
 #ifdef HTEX
-extern	int	open_www_file ARGS((void));
-extern	void	htex_can_it ARGS((void));
+extern	FILE	*xfopen_local ARGS((char *, OPEN_MODE_ARGS));
+#else
+#define xfopen_local	xfopen
+#endif
+
+extern	int	xpipe ARGS((int *));
+extern	DIR	*xopendir ARGS((_Xconst char *));
+extern	_Xconst	struct passwd *ff_getpw ARGS((_Xconst char **, _Xconst char *));
+extern	unsigned long	num ARGS((FILE *, int));
+extern	long	snum ARGS((FILE *, int));
+extern	size_t	property_get_data ARGS((Window, Atom, unsigned char **,
+		  int (*property_get_data) (Display *, Window, Atom, long,
+		  long, Bool, Atom, Atom *, int *, unsigned long *,
+		  unsigned long *, unsigned char **)));
+#if PS
+extern	int	xdvi_temp_fd ARGS((char **));
+#endif
+extern	void	read_PK_index ARGS((struct font *, wide_bool));
+extern	void	read_GF_index ARGS((struct font *, wide_bool));
+#ifdef Omega
+extern	unsigned long read_VF_index ARGS((struct font *, wide_bool));
+#else /* OMEGA */
+extern	void	read_VF_index ARGS((struct font *, wide_bool));
+#endif /* OMEGA */
+
+/* SU 2000/03/10: removed the HTEX condition for next declaration */
+extern	int	pointerlocate ARGS((int *, int *));
+
+#ifdef HTEX
+char *urlocalize P1H(char *filename);
+extern int lastwwwopen;
+
+EXTERN	int	HTeXnext_extern; /* Bring up next file in new xdvi window */
+EXTERN	char	*anchor_name	INIT(NULL);
+EXTERN	Boolean	URL_aware	INIT(False);
+EXTERN	Boolean	highlight	INIT(False);
+extern int HTeXAnestlevel;	/* Hypertext nesting level */
+extern int HTeXSrc;	/* Hypertext nesting level */
+extern int HTeXreflevel;	/* flag for whether we are inside an href */
+
+/* SU 2000/03/10: removed some functions only needed to be known in hypertex.c */
+extern  char    *htex_file_at_index ARGS((int));
+extern  char    *htex_url_at_index ARGS((int));
+extern	int	open_www_file ARGS((int));
 extern	void	search_callback ARGS((Widget, XtPointer, XtPointer));
 extern	void	detach_anchor ARGS((void));
 extern	char	*MyStrAllocCopy ARGS((char **, char *));
-extern	int	pointerlocate ARGS((int *, int *));
+extern  void	htex_open_dvi_file ARGS((void));
 extern	void	htex_recordbits ARGS((int, int, int, int));
 extern	void	htex_initpage ARGS((void));
 extern	void	htex_donepage ARGS((int, int));
 extern	void	htex_parsepages ARGS((void));
 extern	void	htex_parse_page ARGS((int));
 extern	void	htex_scanpage ARGS((int));
-extern	void	htex_scanpage ARGS((int));
 extern	void	htex_dospecial ARGS((long, int));
 extern	void	htex_reinit ARGS((void));
 extern	void	htex_do_loc ARGS((char *));
@@ -946,21 +1246,28 @@ extern	void	add_search ARGS((char *, int));
 extern	int	htex_handleref ARGS((int, int, int));
 extern	void	htex_displayanchor ARGS((int, int, int));
 extern	void	htex_goback ARGS((void));
-extern	int	checkHyperTeX ARGS((char *, int));
-extern	void	htex_handletag ARGS((char *, int));
-extern	void	htex_anchor ARGS((int, char *, int));
-extern	void	htex_dohref ARGS((char *));
-extern	void	htex_drawboxes ARGS((void));
-extern	void	htex_to_page ARGS((int));
-extern	void	htex_to_anchor ARGS((int, int));
+extern	int	checkHyperSrcTeX ARGS((char *, int));
 extern	int	htex_is_url ARGS((const char *));
-extern	void	htex_do_url ARGS((char *));
 extern  void 	paint_anchor ARGS((char *));
-extern	int	fetch_relative_url ARGS((char *, const char *, char *));
+extern	int	fetch_relative_url ARGS((char *, const char *));
 extern	void	wait_for_urls ARGS((void));
-char *figure_mime_type ARGS((char *));
+extern  char	*figure_mime_type ARGS((char *));
+extern	void	htex_cleanup ARGS((void));
 #endif
 
+#ifdef TOOLKIT
+extern void show_help ARGS((Widget));
+extern void do_popup_message PVAR3H(int type, char *helptext, char *format);
+#define MSG_HELP 0
+#define MSG_INFO 1
+#define MSG_WARN 2
+#define MSG_ERR  3
+#else
+extern void show_help ARGS((void));
+#endif /* TOOLKIT */
+
+
+/* END CHUNK xdvi.h 7 */
 #if !HAVE_TEMPNAM
 char *tempnam ARGS((char *, char *));
 #endif
@@ -974,18 +1281,4 @@ _XFUNCPROTOEND
 #define four(fp)	num (fp, 4)
 #define sfour(fp)	snum(fp, 4)
 
-#ifdef HTEX
-/* From xhdvi/url.h */
-/* Some URL stuff: */
-typedef struct {
-	char *url;    /* Full address of the URL we have locally */
-	char *file;   /* Local file name it is stored as */
-} FiletoURLconv;
-
-#define FILELISTCHUNK 20
-EXTERN FiletoURLconv *filelist INIT(NULL);
-EXTERN int nURLs INIT(0);
-
-#endif /* HTEX */
-
 #endif	/* XDVI_H */
